#include "ADSPCM41x_Flash.h"
#include "stdio.h"

/****************************************************************************************************************************
*	Function name:  CM41xFlash_ResetController
*	Description:	This function resets the Flash controller.
*                       - If Flash controller is in sleep mode, this function wakes it up
*                       - If Flash controller is in deep power down mode, this function wakes it up
*                       - If Flash controller is stuck in program mode, this function aborts this operation.
*                       - If Flash controller is stuck in command mode, this function exits this mode.
*                       - while returning, this function clears control word, clears status bits and clears command register

*	Parameter               | Description                       | Valid values                              |
*                           | Flash controller to be reset      | FLASH_CONTROLLER_A or FLASH_CONTROLLER_A
*   Return value:    None.
****************************************************************************************************************************/

__ramfunc  void CM41xFlash_ResetController(unsigned char Flash_Controller) 
{
  unsigned int Flash_Status; 
    if(Flash_Controller == FLASH_CONTROLLER_A)
      Flash_Status = *pREG_FLC0_STAT;
    else if(Flash_Controller == FLASH_CONTROLLER_B)
      Flash_Status = *pREG_FLC1_STAT;  
    else
      Error_function();
        
    // If Flash controller is in sleep mode
    if (Flash_Status & BITM_FLC_STAT_SLP) 
    {
      // if flash controller is not in command mode, send the enter command mode
      CM41xFlash_EnterCommandMode(Flash_Controller);
            
      //  send the sleep wake-up command
      CM41xFlash_IssueCommand(Flash_Controller, SLEEP_WAKEUP_CMD, POLL_COMMAND_STATUS); 

      //  exit command mode
      CM41xFlash_ExitCommandMode(Flash_Controller);
    }
          
    // If flash is in deep power down mode
    if (Flash_Status & BITM_FLC_STAT_PDN) 
    {
      // if flash controller is not in command mode, send the enter command mode
      CM41xFlash_EnterCommandMode(Flash_Controller);
            
      //  send the deep power down wake-up command
      CM41xFlash_IssueCommand(Flash_Controller, POWER_DOWN_WAKEUP_CMD, POLL_COMMAND_STATUS); 

      //  exit command mode
      CM41xFlash_ExitCommandMode(Flash_Controller);
    }

    // If flash is in program/erase mode
    if (Flash_Status & BITM_FLC_STAT_PGMM) 
    {
      // if flash controller is not in command mode, send the enter command mode
      CM41xFlash_EnterCommandMode(Flash_Controller);
            
      //  send the program end command
      CM41xFlash_IssueCommand(Flash_Controller, END_PROGRAM_CMD, POLL_COMMAND_STATUS); 

      //  exit command mode
      CM41xFlash_ExitCommandMode(Flash_Controller);
    }

    // exit command mode, if any
    if (Flash_Status & BITM_FLC_STAT_CMDM) 
      CM41xFlash_ExitCommandMode(Flash_Controller);
          
    // clear COMMAND register
    if(Flash_Controller == FLASH_CONTROLLER_A)
    {
        *pREG_FLC0_CTL   &= ~BITM_FLC_CTL_CMDM;
        *pREG_FLC0_CMD    = 0;
        *pREG_FLC0_STAT   = 0xFFFFFFFF;
        *pREG_FLC0_ADDR   = 0;
        *pREG_FLC0_DATA0  = 0;
        *pREG_FLC0_DATA1  = 0;
        *pREG_FLC0_ECC    = 0;
        *pREG_FLC0_IMSK   = BITM_FLC_IMSK_MBERRF | BITM_FLC_IMSK_MBERRD | BITM_FLC_IMSK_MBERRC | BITM_FLC_IMSK_IFL;
    } 
    else if(Flash_Controller == FLASH_CONTROLLER_B)
    {
        *pREG_FLC1_CTL   &= ~BITM_FLC_CTL_CMDM;
        *pREG_FLC1_CMD    = 0;
        *pREG_FLC1_STAT   = 0xFFFFFFFF;
        *pREG_FLC1_ADDR   = 0;
        *pREG_FLC1_DATA0  = 0;
        *pREG_FLC1_DATA1  = 0;
        *pREG_FLC1_ECC    = 0;
        *pREG_FLC1_IMSK   = BITM_FLC_IMSK_MBERRF | BITM_FLC_IMSK_MBERRD | BITM_FLC_IMSK_MBERRC | BITM_FLC_IMSK_IFL;
    }
    else
      Error_function();
}  


/****************************************************************************************************************************
*	Function name:  CM41xFlash_ErasePage
*	Description:	This function erases the 4KB block of flash
*	Parameters:
*                   Flash_Controller: Flash controller to be reset.  Valid values: FLASH_CONTROLLER_A or FLASH_CONTROLLER_A
*                   Page_ADDR: Address of page to be erased.       
*                              Valid values: FLASH_A      - 0x1100_0000 to 0x1107_F000....in the 0x1000 steps
*                                            FLASH_B      - 0x1108_0000 to 0x110F_F000....in the 0x1000 steps
*                                            Info Block A - 0x1180_0000
*                                            Info Block B - 0x1188_0000
*   Return value:    None.
****************************************************************************************************************************/

__ramfunc  void CM41xFlash_ErasePage(unsigned char Flash_Controller, unsigned int Page_ADDR)
{
    unsigned char IRFWEN, IRFREN;

    CM41xFlash_SetInfoBlockProp(Flash_Controller, Page_ADDR, &IRFWEN, &IRFREN);
              
    CM41xFlash_EnterCommandMode(Flash_Controller);
            
    // send the page erase command
    if(Flash_Controller == FLASH_CONTROLLER_A)
      *pREG_FLC0_ADDR = Page_ADDR ; 
    else if(Flash_Controller == FLASH_CONTROLLER_B)
      *pREG_FLC1_ADDR = Page_ADDR ;
    else
      Error_function();
    
    CM41xFlash_IssueCommand(Flash_Controller, PAGE_ERASE_CMD, POLL_COMMAND_STATUS); 

    //  exxit command mode
    CM41xFlash_ExitCommandMode(Flash_Controller);
//----------------------------------------------------------------------------------------------    
    
    unsigned int *pPage_ADDR = (unsigned int*)Page_ADDR;
 
    for(int loop_count = 0; loop_count < (FLASH_PAGE_SIZ/4); loop_count++)
    {
      if(*pPage_ADDR++ != 0xFFFFFFFF)
        Error_function();
    }
//----------------------------------------------------------------------------------------------    

    // restore IRFWEN bit
    CM41xFlash_RestoreInfoBlockProp(Flash_Controller, IRFWEN, IRFREN);  
}


/****************************************************************************************************************************
*	Function name:  CM41xFlash_MassErase
*	Description:	This function erases whole flash including or excludibg Info Block
*	Parameters:
*                   Flash_Controller: Flash controller to be reset.  Valid values: FLASH_CONTROLLER_A or FLASH_CONTROLLER_A
*                   Erase_Info_Block: Mass erase should include Info Block or not.  1: Erase with Info block  0: Exclude Info Block
*   Return value:    None.
****************************************************************************************************************************/
int loop_count;
__ramfunc  void CM41xFlash_MassErase(unsigned char Flash_Controller, unsigned char Erase_Info_Block)
{
    unsigned char IRFWEN, IRFREN;

    if (Erase_Info_Block == 0)
    {
        if(Flash_Controller == FLASH_CONTROLLER_A)
		{
              CM41xFlash_SetInfoBlockProp(Flash_Controller, FLASHA_BASE_ADDR, &IRFWEN, &IRFREN);
			  *pREG_FLC0_ADDR = FLASHA_BASE_ADDR;
        }
        else if(Flash_Controller == FLASH_CONTROLLER_B)
		{
              CM41xFlash_SetInfoBlockProp(Flash_Controller, FLASHB_BASE_ADDR, &IRFWEN, &IRFREN);
			  *pREG_FLC1_ADDR = FLASHB_BASE_ADDR;
        }
        else
            Error_function();
    }
    else if(Erase_Info_Block == 1)
    {
        if(Flash_Controller == FLASH_CONTROLLER_A)
		{
            CM41xFlash_SetInfoBlockProp(Flash_Controller, FLASHA_INFO_BLOCK_ADDR, &IRFWEN, &IRFREN);
			*pREG_FLC0_ADDR = FLASHA_INFO_BLOCK_ADDR;
        }
        else if(Flash_Controller == FLASH_CONTROLLER_B)
		{
            CM41xFlash_SetInfoBlockProp(Flash_Controller, FLASHB_INFO_BLOCK_ADDR, &IRFWEN, &IRFREN);
			*pREG_FLC1_ADDR = FLASHB_INFO_BLOCK_ADDR;
        }
        else
            Error_function();      
    }
   else
            Error_function();
          
    // if flash controller is not in command mode, send the enter command mode
        CM41xFlash_EnterCommandMode(Flash_Controller);
              
    // send the Mass Erase command
        CM41xFlash_IssueCommand(Flash_Controller, MASS_ERASE_CMD, POLL_COMMAND_STATUS); 

    //  exit command mode
        CM41xFlash_ExitCommandMode(Flash_Controller);
//----------------------------------------------------------------------------------------------    
    
    unsigned int *pFlash_ADDR;
    
    if(Flash_Controller == FLASH_CONTROLLER_A)
        pFlash_ADDR = (unsigned int*) FLASHA_BASE_ADDR;
    else if(Flash_Controller == FLASH_CONTROLLER_B)
        pFlash_ADDR = (unsigned int*) FLASHB_BASE_ADDR;
 /*
    for(loop_count = 0; loop_count < ((FLASH_PAGE_SIZ/4) * FLASH_PAGE_COUNT); loop_count++)
    {
      if(*pFlash_ADDR++ != 0xFFFFFFFF)
        Error_function();
    }
   */ 
    if(Erase_Info_Block == 1)
    {
       if(Flash_Controller == FLASH_CONTROLLER_A)
            pFlash_ADDR = (unsigned int*) FLASHA_INFO_BLOCK_ADDR;
        else if(Flash_Controller == FLASH_CONTROLLER_B)
            pFlash_ADDR = (unsigned int*) FLASHB_INFO_BLOCK_ADDR;

        for(loop_count = 0; loop_count < (FLASH_PAGE_SIZ/4); loop_count++)
        {
          if(*pFlash_ADDR++ != 0xFFFFFFFF)
            Error_function();
        }
    }    
//----------------------------------------------------------------------------------------------    

    // restore IRFWEN bit
        CM41xFlash_RestoreInfoBlockProp(Flash_Controller, IRFWEN, IRFREN);
}


/****************************************************************************************************************************
*	Function name:  CM41xFlash_ReadBuffer
*	Description:	This function reads from Flash. It reads in normal mode i.e. not in command mode
*	Parameters:
*                   Flash_Controller: Flash controller to be reset.  Valid values: FLASH_CONTROLLER_A or FLASH_CONTROLLER_A
*                   Flash_ADDR: Address of flash to be read.       
*                              Valid values: FLASH_A      - 0x1100_0000 to 0x1107_FFFF
*                                            FLASH_B      - 0x1108_0000 to 0x110F_FFFF
*                                            Info Block A - 0x1180_0000 to 0x1180_0FFF
*                                            Info Block B - 0x1188_0000 to 0x1188_0FFF
*                   Read_count: no of 32-bit words to be read
*                   Read_Buffer: pointer to buffer in which read data to be stored.
*   Return value:    None.
****************************************************************************************************************************/

__ramfunc  void CM41xFlash_ReadBuffer(unsigned char Flash_Controller, unsigned int Flash_ADDR, unsigned int Read_count, unsigned int* Read_Buffer)
{
    unsigned char IRFWEN, IRFREN;
    unsigned int *pFlash_ADDR = (unsigned int*)Flash_ADDR;

    CM41xFlash_ExitCommandMode(Flash_Controller);

    CM41xFlash_SetInfoBlockProp(Flash_Controller, Flash_ADDR, &IRFWEN, &IRFREN);
 
    for(unsigned int loop_count = 0; loop_count < Read_count; loop_count++)
        *Read_Buffer++ = *pFlash_ADDR++;   

    CM41xFlash_RestoreInfoBlockProp(Flash_Controller, IRFWEN, IRFREN);
}


/****************************************************************************************************************************
*	Function name:  CM41xFlash_WriteBuffer
*	Description:	This function writes/programs the Flash. 
*	Parameters:
*                   Flash_Controller: Flash controller to be reset.  Valid values: FLASH_CONTROLLER_A or FLASH_CONTROLLER_A
*                   Flash_ADDR: Address of flash to be written. 
*                              Valid values: FLASH_A      - 0x1100_0000 to 0x1107_FFFF
*                                            FLASH_B      - 0x1108_0000 to 0x110F_FFFF
*                                            Info Block A - 0x1180_0000 to 0x1180_0FFF
*                                            Info Block B - 0x1188_0000 to 0x1188_0FFF
*                   Read_count: no of 32-bit words to be written
*                   Read_Buffer: pointer to data buffer to be written.
*   Return value:    None.
****************************************************************************************************************************/

__ramfunc  void CM41xFlash_WriteBuffer(unsigned char Flash_Controller, unsigned int Flash_ADDR, unsigned int Program_count, unsigned int* Write_Buffer)
{
  unsigned char IRFWEN, IRFREN;
    CM41xFlash_SetInfoBlockProp(Flash_Controller, Flash_ADDR, &IRFWEN, &IRFREN);
 
    unsigned int count = 4 * Program_count; 

    if(Flash_ADDR % FLASH_ROW_SIZE)
    {
        unsigned int temp_count = FLASH_ROW_SIZE - (Flash_ADDR & 0x1FC);

        if(temp_count >= count)
        {
          while(count > 0)
          {
            CM41xFlash_WriteWord(Flash_Controller, Flash_ADDR, *Write_Buffer);
            Flash_ADDR += 4;
            Write_Buffer++;
            count  -= 4;
          }
        }
        else
        {
          count -= temp_count;
          while(temp_count > 0)
          {
            CM41xFlash_WriteWord(Flash_Controller, Flash_ADDR, *Write_Buffer);
            Flash_ADDR += 4;
            Write_Buffer++;
            temp_count  -= 4;
          }
        }
    }
    
    while((count >= FLASH_ROW_SIZE))
    {
        CM41xFlash_WriteROW(Flash_Controller, Flash_ADDR, Write_Buffer);
        Flash_ADDR += FLASH_ROW_SIZE;
        Write_Buffer += (FLASH_ROW_SIZE/4);
        count  -= FLASH_ROW_SIZE;
    }

    while(count > 0)
    {
        CM41xFlash_WriteWord(Flash_Controller, Flash_ADDR, *Write_Buffer);
        Flash_ADDR += 4;
        Write_Buffer++;
        count  -= 4;
    }
    
    CM41xFlash_ExitCommandMode(Flash_Controller);
//----------------------------------------------------------------------------------------------    
    
    unsigned int *pFlash_ADDR = (unsigned int*)Flash_ADDR - Program_count;
 
    Write_Buffer-= Program_count;
    for(int loop_count = 0; loop_count < Program_count; loop_count++)
    {
      if(*pFlash_ADDR++ != *Write_Buffer++)
        Error_function();
    }
//----------------------------------------------------------------------------------------------    
    
    CM41xFlash_RestoreInfoBlockProp(Flash_Controller, IRFWEN, IRFREN);
}



unsigned char ECC_P0[26] = {0, 1,  2,  3,  4,  5,  6,  7,  8,  9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20, 56, 57, 59, 60, 62};
unsigned char ECC_P1[26] = {0, 1,  2,  3,  4,  5, 21, 22, 23, 24, 25, 26, 27, 28, 29, 30, 31, 32, 33, 34, 35, 56, 57, 59, 61, 62};
unsigned char ECC_P2[26] = {0, 6,  7,  8,  9, 10, 21, 22, 23, 24, 25, 36, 37, 38, 39, 40, 41, 42, 43, 44, 45, 56, 58, 59, 61, 62};
unsigned char ECC_P3[26] = {1, 6, 11, 12, 13, 14, 21, 26, 27, 28, 29, 36, 37, 38, 39, 46, 47, 48, 49, 50, 51, 56, 58, 59, 61, 63};
unsigned char ECC_P4[26] = {2, 7, 11, 15, 16, 17, 22, 26, 30, 31, 32, 36, 40, 41, 42, 46, 47, 48, 52, 53, 54, 56, 58, 60, 61, 63};
unsigned char ECC_P5[26] = {3, 8, 12, 15, 18, 19, 23, 27, 30, 33, 34, 37, 40, 43, 44, 46, 49, 50, 52, 53, 55, 57, 58, 60, 61, 63};
unsigned char ECC_P6[26] = {4, 9, 13, 16, 18, 20, 24, 28, 31, 33, 35, 38, 41, 43, 45, 47, 49, 51, 52, 54, 55, 57, 58, 60, 62, 63};
unsigned char ECC_P7[26] = {5,10, 14, 17, 19, 20, 25, 29, 32, 34, 35, 39, 42, 44, 45, 48, 50, 51, 53, 54, 55, 57, 59, 60, 62, 63};

__ramfunc  unsigned char CM41xFlash_CalcECC(unsigned int Data0, unsigned int Data1)
{
    unsigned char P0=1, P1=1, P2=1, P3=1, P4=1, P5=1, P6=1, P7=1, count, ECC=0;

    for(count=0; count<sizeof(ECC_P0); count++)
    {
      if(ECC_P0[count] < 32)
        P0 ^= ((Data0 >> ECC_P0[count]) & 0x1);
      else
        P0 ^= ((Data1 >> (ECC_P0[count] - 32)) & 0x1);
    }
    
    for(count=0; count<sizeof(ECC_P1); count++)
    {
      if(ECC_P1[count] < 32)
        P1 ^= ((Data0 >> ECC_P1[count]) & 0x1);
      else
        P1 ^= ((Data1 >> (ECC_P1[count] - 32)) & 0x1);
    }
    
    for(count=0; count<sizeof(ECC_P2); count++)
    {
      if(ECC_P2[count] < 32)
        P2 ^= ((Data0 >> ECC_P2[count]) & 0x1);
      else
        P2 ^= ((Data1 >> (ECC_P2[count] - 32)) & 0x1);
    }
    
    for(count=0; count<sizeof(ECC_P3); count++)
    {
      if(ECC_P3[count] < 32)
        P3 ^= ((Data0 >> ECC_P3[count]) & 0x1);
      else
        P3 ^= ((Data1 >> (ECC_P3[count] - 32)) & 0x1);
    }
     
    for(count=0; count<sizeof(ECC_P4); count++)
    {
      if(ECC_P4[count] < 32)
        P4 ^= ((Data0 >> ECC_P4[count]) & 0x1);
      else
        P4 ^= ((Data1 >> (ECC_P4[count] - 32)) & 0x1);
    }
    
    for(count=0; count<sizeof(ECC_P5); count++)
    {
      if(ECC_P5[count] < 32)
        P5 ^= ((Data0 >> ECC_P5[count]) & 0x1);
      else
        P5 ^= ((Data1 >> (ECC_P5[count] - 32)) & 0x1);
    }
    
    for(count=0; count<sizeof(ECC_P6); count++)
    {
      if(ECC_P6[count] < 32)
        P6 ^= ((Data0 >> ECC_P6[count]) & 0x1);
      else
        P6 ^= ((Data1 >> (ECC_P6[count] - 32)) & 0x1);
    }
    
    for(count=0; count<sizeof(ECC_P7); count++)
    {
      if(ECC_P7[count] < 32)
        P7 ^= ((Data0 >> ECC_P7[count]) & 0x1);
      else
        P7 ^= ((Data1 >> (ECC_P7[count] - 32)) & 0x1);
    }   
    ECC = P0 | (P1 << 1) | (P2 << 2) | (P3 << 3) | (P4 << 4) | (P5 << 5) | (P6 << 6) | (P7 << 7) ; 
    return ECC;
}


/****************************************************************************************************************************
/****************************************************************************************************************************
*
* Helper functions: 
*
/****************************************************************************************************************************
/****************************************************************************************************************************


/*===========================================================================================================================
*	Function name:  CM41xFlash_EnterCommandMode
*	Description:	Enter the flash controller in command mode
*	Parameter               | Description                                       | Valid values                              |
*                           | Flash controller to be entered into command mode  | FLASH_CONTROLLER_A or FLASH_CONTROLLER_A
*   Return value:    None.
============================================================================================================================*/

__ramfunc  void CM41xFlash_EnterCommandMode(unsigned char Flash_Controller) 
{
    switch (Flash_Controller)
    {
        case FLASH_CONTROLLER_A:
                  
          if (!(*pREG_FLC0_STAT & BITM_FLC_STAT_CMDM)) 
          {
            // set the command mode CONTROL bit
                *pREG_FLC0_CTL |= BITM_FLC_CTL_CMDM;

            // write the key sequence
                *pREG_FLC0_CMD_KEY = FLC_CMD_KEY0;
                *pREG_FLC0_CMD_KEY = FLC_CMD_KEY1;

            // make sure that Flash controller is in command mode
                while ((*pREG_FLC0_STAT & BITM_FLC_STAT_CMDM) == 0);

            // verify no error status
                if (*pREG_FLC0_STAT & FLASH_ERROR_STATUS_BITS)
                    Error_function();
          }       
        break;
    
        case FLASH_CONTROLLER_B:

          if (!(*pREG_FLC1_STAT & BITM_FLC_STAT_CMDM)) 
          {

            // set the command mode CONTROL bit
                *pREG_FLC1_CTL |= BITM_FLC_CTL_CMDM;

            // write the key sequence
                *pREG_FLC1_CMD_KEY = FLC_CMD_KEY0;
                *pREG_FLC1_CMD_KEY = FLC_CMD_KEY1;

            // make sure that Flash controller is in command mode
                while ((*pREG_FLC1_STAT & BITM_FLC_STAT_CMDM) == 0);

            // verify no error status
                if (*pREG_FLC1_STAT & FLASH_ERROR_STATUS_BITS)
                    Error_function();
          }
        break;
        
        default: 
                printf("\nInvalid Flash Controller ID\n");
                Error_function();
        break;
    }
}


/*===========================================================================================================================
*	Function name:  CM41xFlash_ExitCommandMode
*	Description:	Exits the flash controller in command mode
*	Parameter               | Description                                       | Valid values                              |
*                           | Flash controller to be exited from command mode   | FLASH_CONTROLLER_A or FLASH_CONTROLLER_A
*   Return value:    None.
============================================================================================================================*/

__ramfunc  void CM41xFlash_ExitCommandMode(unsigned char Flash_Controller) 
{
    switch (Flash_Controller)
    {
        case FLASH_CONTROLLER_A:

          if (*pREG_FLC0_STAT & BITM_FLC_STAT_CMDM) 
          {   
            // clear the command mode CONTROL bit
                *pREG_FLC0_CTL &= ~BITM_FLC_CTL_CMDM;

            // make sure that Flash controller is out of command mode
                while ((*pREG_FLC0_STAT & BITM_FLC_STAT_CMDM) == BITM_FLC_STAT_CMDM);

            // verify no error status
                if (*pREG_FLC0_STAT & FLASH_ERROR_STATUS_BITS)
                    Error_function();
          }
        break;
    
        case FLASH_CONTROLLER_B:
    
          if (*pREG_FLC1_STAT & BITM_FLC_STAT_CMDM) 
          {
            // set the command mode CONTROL bit
                *pREG_FLC1_CTL &= ~BITM_FLC_CTL_CMDM;

            // make sure that Flash controller is out of command mode
                while ((*pREG_FLC1_STAT & BITM_FLC_STAT_CMDM) == BITM_FLC_STAT_CMDM);

            // verify no error status
                if (*pREG_FLC1_STAT & FLASH_ERROR_STATUS_BITS)
                    Error_function();
          }
        break;
        
        default: 
                printf("\nInvalid Flash Controller ID\n");
                Error_function();
        break;
    }
}


/*===========================================================================================================================
*	Function name:  CM41xFlash_IssueCommand
*	Description:	Issue the commands to flash controller
*	Parameter                               | Description                                       | Valid values                              |
*                   Flash_Controller        | Flash controller to be entered into command mode  | FLASH_CONTROLLER_A or FLASH_CONTROLLER_A
*                   command                 | command to be send                                | Refer ADSPCM41x_Flash.h file for the commands
*                   poll_status             | If status polling is needed after issuing commands| DO_NOT_POLL_COMMAND_STATUS    POLL_COMMAND_STATUS
*   Return value:    None.
============================================================================================================================*/

__ramfunc  void CM41xFlash_IssueCommand(unsigned char Flash_Controller, unsigned char command, unsigned char poll_status) 
{
    switch(Flash_Controller)
    {
        case FLASH_CONTROLLER_A:
            
            switch (command)
            {
                case PAGE_ERASE_CMD:
                  // Issue Page Erase command
                  *pREG_FLC0_CMD = ENUM_FLC_CMD_PER; 
				  
                  // check for status if instructed
                  if (poll_status == POLL_COMMAND_STATUS)
                  {
                      while ((*pREG_FLC0_STAT & BITM_FLC_STAT_PERP) == 0);
                      while ((*pREG_FLC0_STAT & BITM_FLC_STAT_PERP) == BITM_FLC_STAT_PERP);
                  }
                break;

                case MASS_ERASE_CMD:
                  // Issue Mass Erase command
                  *pREG_FLC0_CMD = ENUM_FLC_CMD_MER; 
				  
                  // check for status if instructed
                  if (poll_status == POLL_COMMAND_STATUS)
                  {
                        while ((*pREG_FLC0_STAT & BITM_FLC_STAT_MERP) == 0);
                    while ((*pREG_FLC0_STAT & BITM_FLC_STAT_MERP) == BITM_FLC_STAT_MERP);
                  }
                break;
                  
                case START_PROGRAM_CMD:
                  // Issue Start Program command
                  *pREG_FLC0_CMD = ENUM_FLC_CMD_SPGMM;
 
                  // check for status if instructed
                  if (poll_status == POLL_COMMAND_STATUS)
                  {
                        while ((*pREG_FLC0_STAT & BITM_FLC_STAT_PGMMP) == 0);	//	wait till controller goes to program mode change state
                        while ((*pREG_FLC0_STAT & BITM_FLC_STAT_PGMMP) == BITM_FLC_STAT_PGMMP);	//	wait till controller is in program mode change state
                        
                    while ((*pREG_FLC0_STAT & BITM_FLC_STAT_PGMM) == 0);	//	wait till controller goes to program mode
                  }
                break;
	
                case  END_PROGRAM_CMD:
                  // Issue Start Program command
                  *pREG_FLC0_CMD = ENUM_FLC_CMD_EPGMM;
 
                  // check for status if instructed
                  if (poll_status == POLL_COMMAND_STATUS)
                  {
                        while ((*pREG_FLC0_STAT & BITM_FLC_STAT_PGMMP) == 0);	//	wait till controller goes to program mode change state
                        while ((*pREG_FLC0_STAT & BITM_FLC_STAT_PGMMP) == BITM_FLC_STAT_PGMMP);	//	wait till controller is in program mode change state
                        
                    while ((*pREG_FLC0_STAT & BITM_FLC_STAT_PGMM) == BITM_FLC_STAT_PGMM);	//	wait till controller is in program mode
                  }
                break;
				
                case PROGRAM_CMD:
                  // Issue Start Program command
                        *pREG_FLC0_CMD = ENUM_FLC_CMD_PGMC;
                        
                  // check for status if instructed
                  if (poll_status == POLL_COMMAND_STATUS)
                  {
                        while ((*pREG_FLC0_STAT & BITM_FLC_STAT_PGMCP) == 0);	//	wait till program operations starts
                        while ((*pREG_FLC0_STAT & BITM_FLC_STAT_PGMCP) == BITM_FLC_STAT_PGMCP);	//	wait till program operation ends
                  }
                break;
				
                case READ_CMD:
                  // Issue Read command
                  *pREG_FLC0_CMD = ENUM_FLC_CMD_RDC;
                
                  // check for status if instructed
                  if (poll_status == POLL_COMMAND_STATUS)
                  {
                  //	while ((*pREG_FLC0_STAT & BITM_FLC_STAT_RDCP) == 0);	//	wait till read operations starts
                        while ((*pREG_FLC0_STAT & BITM_FLC_STAT_RDCP) == BITM_FLC_STAT_RDCP);	//	wait till read operation ends
                  }
                break;
                
                case SLEEP_CMD:
                  // Issue Read command
                  *pREG_FLC0_CMD = ENUM_FLC_CMD_SLPC;
                
                  // check for status if instructed
                  if (poll_status == POLL_COMMAND_STATUS)
                  {
//                        while ((*pREG_FLC0_STAT & BITM_FLC_STAT_SLPCP) == 0);	//	wait till controller goes to sleep command pending mode
                        while ((*pREG_FLC0_STAT & BITM_FLC_STAT_SLPCP) == BITM_FLC_STAT_SLPCP);	//	wait till controller comes out of sleep command pending
                        
                    while ((*pREG_FLC0_STAT & BITM_FLC_STAT_SLP) == 0);		//	wait till controller goes to sleep mode
                  }
                break;
                
                case SLEEP_WAKEUP_CMD:
                  // Issue Read command
                  *pREG_FLC0_CMD = ENUM_FLC_CMD_WKSL;
                
                  // check for status if instructed
                  if (poll_status == POLL_COMMAND_STATUS)
                  {
//                        while ((*pREG_FLC0_STAT & BITM_FLC_STAT_WKUPP) == 0);	//	wait till controller goes to sleep wakeup command pending mode
                        while ((*pREG_FLC0_STAT & BITM_FLC_STAT_WKUPP) == BITM_FLC_STAT_WKUPP);	//	wait till controller comes out of sleep wakeup command pending
                        
                    while ((*pREG_FLC0_STAT & BITM_FLC_STAT_SLP) == BITM_FLC_STAT_SLP);		//	wait till controller is in sleep mode
                  }
                break;
                
                case POWER_DOWN_CMD:
                  // Issue Read command
                  *pREG_FLC0_CMD = ENUM_FLC_CMD_PDNC;
                
                  // check for status if instructed
                  if (poll_status == POLL_COMMAND_STATUS)
                  {
//                        while ((*pREG_FLC0_STAT & BITM_FLC_STAT_PDNCP) == 0);	//	wait till controller goes to power-down command pending mode
                        while ((*pREG_FLC0_STAT & BITM_FLC_STAT_PDNCP) == BITM_FLC_STAT_PDNCP);	//	wait till controller comes out of power-down command pending
                        
                    while ((*pREG_FLC0_STAT & BITM_FLC_STAT_PDN) == BITM_FLC_STAT_PDN);		//	wait till controller goes to power-down mode
                  }
                break;
                
                case POWER_DOWN_WAKEUP_CMD:
                  // Issue Read command
                  *pREG_FLC0_CMD = ENUM_FLC_CMD_WKPD;
                
                  // check for status if instructed
                  if (poll_status == POLL_COMMAND_STATUS)
                  {
                        while ((*pREG_FLC0_STAT & BITM_FLC_STAT_WKUPP) == 0);	//	wait till controller goes to power-down wakeup command pending mode
                        while ((*pREG_FLC0_STAT & BITM_FLC_STAT_WKUPP) == BITM_FLC_STAT_WKUPP);	//	wait till controller comes out of power-down wakeup command pending
                        
                    while ((*pREG_FLC0_STAT & BITM_FLC_STAT_PDN) == BITM_FLC_STAT_PDN);		//	wait till controller is in power-down mode
                  }
                break;
                
                case UPDATE_TIMING_INFO:
                  // Issue Read command
                  *pREG_FLC0_CMD = ENUM_FLC_CMD_UTM;
                
                // check for status if instructed
                  if (poll_status == POLL_COMMAND_STATUS)
                  {
                        while ((*pREG_FLC0_STAT & BITM_FLC_STAT_UTMP) == 0);	//	wait till controller goes to Update Timing info command pending mode
                        while ((*pREG_FLC0_STAT & BITM_FLC_STAT_UTMP) == BITM_FLC_STAT_UTMP);	//	wait till controller comes out of Update Timing info command pending
                  }
                
                break;
                
                case ABORT_CMD:
                  // Issue Read command
                  *pREG_FLC0_CMD = ENUM_FLC_CMD_ABORT;
                
                  // check for status if instructed
                  if (poll_status == POLL_COMMAND_STATUS)
                  {
                        while ((*pREG_FLC0_STAT & BITM_FLC_STAT_ABTP) == 0);	//	wait till controller goes to abort command pending mode
                        while ((*pREG_FLC0_STAT & BITM_FLC_STAT_ABTP) == BITM_FLC_STAT_ABTP);	//	wait till controller comes out of abort command pending
                    
                        while ((*pREG_FLC0_STAT & BITM_FLC_STAT_CMDM) == BITM_FLC_STAT_CMDM);		//	wait till controller is in command mode				
                  }
                break;
        
                default:
                        printf("\nInvalid Flash command ID\n");
                        Error_function();
                break;
            }
                  
            // verify no error status
            if (*pREG_FLC0_STAT & FLASH_ERROR_STATUS_BITS)
                Error_function();
            
            break;
			
	case FLASH_CONTROLLER_B:
            switch (command)
            {
                case PAGE_ERASE_CMD:
                  // Issue Page Erase command
                  *pREG_FLC1_CMD = ENUM_FLC_CMD_PER; 
                  
                  // check for status if instructed
                  if (poll_status == POLL_COMMAND_STATUS)
                  {
                        while ((*pREG_FLC1_STAT & BITM_FLC_STAT_PERP) == 0);
                    while ((*pREG_FLC1_STAT & BITM_FLC_STAT_PERP) == BITM_FLC_STAT_PERP);
                  }
                break;

                case MASS_ERASE_CMD:
                  // Issue Mass Erase command
                  *pREG_FLC1_CMD = ENUM_FLC_CMD_MER; 
				  
                  // check for status if instructed
                  if (poll_status == POLL_COMMAND_STATUS)
                  {
                        while ((*pREG_FLC1_STAT & BITM_FLC_STAT_MERP) == 0);
                    while ((*pREG_FLC1_STAT & BITM_FLC_STAT_MERP) == BITM_FLC_STAT_MERP);
                  }
                break;
                  
                case START_PROGRAM_CMD:
                  // Issue Start Program command
                  *pREG_FLC1_CMD = ENUM_FLC_CMD_SPGMM;
 
                  // check for status if instructed
                  if (poll_status == POLL_COMMAND_STATUS)
                  {
                        while ((*pREG_FLC1_STAT & BITM_FLC_STAT_PGMMP) == 0);	//	wait till controller goes to program mode change state
                        while ((*pREG_FLC1_STAT & BITM_FLC_STAT_PGMMP) == BITM_FLC_STAT_PGMMP);	//	wait till controller is in program mode change state
                        
                    while ((*pREG_FLC1_STAT & BITM_FLC_STAT_PGMM) == 0);	//	wait till controller goes to program mode
                  }
                break;
                
                case  END_PROGRAM_CMD:
                  // Issue Start Program command
                  *pREG_FLC1_CMD = ENUM_FLC_CMD_EPGMM;
 
                  // check for status if instructed
                  if (poll_status == POLL_COMMAND_STATUS)
                  {
                        while ((*pREG_FLC1_STAT & BITM_FLC_STAT_PGMMP) == 0);	//	wait till controller goes to program mode change state
                        while ((*pREG_FLC1_STAT & BITM_FLC_STAT_PGMMP) == BITM_FLC_STAT_PGMMP);	//	wait till controller is in program mode change state
                        
                    while ((*pREG_FLC1_STAT & BITM_FLC_STAT_PGMM) == BITM_FLC_STAT_PGMM);	//	wait till controller is in program mode
                  }
                break;
                
                case PROGRAM_CMD:
                  // Issue Start Program command
                        *pREG_FLC1_CMD = ENUM_FLC_CMD_PGMC;
                        
                  // check for status if instructed
                  if (poll_status == POLL_COMMAND_STATUS)
                  {
                        while ((*pREG_FLC1_STAT & BITM_FLC_STAT_PGMCP) == 0);	//	wait till program operations starts
                        while ((*pREG_FLC1_STAT & BITM_FLC_STAT_PGMCP) == BITM_FLC_STAT_PGMCP);	//	wait till program operation ends
                  }
                break;
                
                case READ_CMD:
                  // Issue Read command
                  *pREG_FLC1_CMD = ENUM_FLC_CMD_RDC;
                
                  // check for status if instructed
                  if (poll_status == POLL_COMMAND_STATUS)
                  {
//                      while ((*pREG_FLC1_STAT & BITM_FLC_STAT_RDCP) == 0);	//	wait till read operations starts
                        while ((*pREG_FLC1_STAT & BITM_FLC_STAT_RDCP) == BITM_FLC_STAT_RDCP);	//	wait till read operation ends
                  }
                break;
                
                case SLEEP_CMD:
                  // Issue Read command
                  *pREG_FLC1_CMD = ENUM_FLC_CMD_SLPC;
                
                  // check for status if instructed
                  if (poll_status == POLL_COMMAND_STATUS)
                  {
//                        while ((*pREG_FLC1_STAT & BITM_FLC_STAT_SLPCP) == 0);	//	wait till controller goes to sleep command pending mode
                        while ((*pREG_FLC1_STAT & BITM_FLC_STAT_SLPCP) == BITM_FLC_STAT_SLPCP);	//	wait till controller comes out of sleep command pending
                        
                    while ((*pREG_FLC1_STAT & BITM_FLC_STAT_SLP) == 0);		//	wait till controller goes to sleep mode
                  }
                break;
                
                case SLEEP_WAKEUP_CMD:
                  // Issue Read command
                  *pREG_FLC1_CMD = ENUM_FLC_CMD_WKSL;
                
                  // check for status if instructed
                  if (poll_status == POLL_COMMAND_STATUS)
                  {
//                        while ((*pREG_FLC1_STAT & BITM_FLC_STAT_WKUPP) == 0);	//	wait till controller goes to sleep wakeup command pending mode
                        while ((*pREG_FLC1_STAT & BITM_FLC_STAT_WKUPP) == BITM_FLC_STAT_WKUPP);	//	wait till controller comes out of sleep wakeup command pending
                        
                    while ((*pREG_FLC1_STAT & BITM_FLC_STAT_SLP) == BITM_FLC_STAT_SLP);		//	wait till controller is in sleep mode
                  }
                break;
                
                case POWER_DOWN_CMD:
                  // Issue Read command
                  *pREG_FLC1_CMD = ENUM_FLC_CMD_PDNC;
                
                  // check for status if instructed
                  if (poll_status == POLL_COMMAND_STATUS)
                  {
//                        while ((*pREG_FLC1_STAT & BITM_FLC_STAT_PDNCP) == 0);	//	wait till controller goes to power-down command pending mode
                        while ((*pREG_FLC1_STAT & BITM_FLC_STAT_PDNCP) == BITM_FLC_STAT_PDNCP);	//	wait till controller comes out of power-down command pending
                        
                    while ((*pREG_FLC1_STAT & BITM_FLC_STAT_PDN) == 0);		//	wait till controller goes to power-down mode
                  }
                break;
                
                case POWER_DOWN_WAKEUP_CMD:
                  // Issue Read command
                  *pREG_FLC1_CMD = ENUM_FLC_CMD_WKPD;
                
                  // check for status if instructed
                  if (poll_status == POLL_COMMAND_STATUS)
                  {
                        while ((*pREG_FLC1_STAT & BITM_FLC_STAT_WKUPP) == 0);	//	wait till controller goes to power-down wakeup command pending mode
                        while ((*pREG_FLC1_STAT & BITM_FLC_STAT_WKUPP) == BITM_FLC_STAT_WKUPP);	//	wait till controller comes out of power-down wakeup command pending
                        
                    while ((*pREG_FLC1_STAT & BITM_FLC_STAT_PDN) == BITM_FLC_STAT_PDN);		//	wait till controller is in power-down mode
                  }
                break;
                
                case UPDATE_TIMING_INFO:
                  // Issue Read command
                  *pREG_FLC1_CMD = ENUM_FLC_CMD_UTM;
                
                // check for status if instructed
                  if (poll_status == POLL_COMMAND_STATUS)
                  {
                        while ((*pREG_FLC1_STAT & BITM_FLC_STAT_UTMP) == 0);	//	wait till controller goes to Update Timing info command pending mode
                        while ((*pREG_FLC1_STAT & BITM_FLC_STAT_UTMP) == BITM_FLC_STAT_UTMP);	//	wait till controller comes out of Update Timing info command pending
                  }
                
                break;
                
                case ABORT_CMD:
                  // Issue Read command
                  *pREG_FLC1_CMD = ENUM_FLC_CMD_ABORT;
                
                  // check for status if instructed
                  if (poll_status == POLL_COMMAND_STATUS)
                  {
                        while ((*pREG_FLC1_STAT & BITM_FLC_STAT_ABTP) == 0);	//	wait till controller goes to abort command pending mode
                        while ((*pREG_FLC1_STAT & BITM_FLC_STAT_ABTP) == BITM_FLC_STAT_ABTP);	//	wait till controller comes out of abort command pending
                    
                        while ((*pREG_FLC1_STAT & BITM_FLC_STAT_CMDM) == BITM_FLC_STAT_CMDM);		//	wait till controller is in command mode				
                  }
                break;
        
                default:
                        printf("\nInvalid Flash command ID\n");
                        Error_function();
                break;
            }
                  
            // verify no error status
            if (*pREG_FLC1_STAT & FLASH_ERROR_STATUS_BITS)
                Error_function();
			
            break;
			
            default:
		printf("\nInvalid Flash Controller ID\n");
		Error_function();
            break;	
	}
}


/*===========================================================================================================================
*	Function name:  CM41xFlash_WritePage
*	Description:	Writes 512 bytes to be programmed
*	Parameter                               | Description                                       | Valid values                              |
*                   Flash_Controller        | Flash controller to be entered into command mode  | FLASH_CONTROLLER_A or FLASH_CONTROLLER_A
*                   ROW_ADDR: Address of page to be written.       
*                              Valid values: FLASH_A      - 0x1100_0000 to 0x1107_FE00....in the 0x200 steps
*                                            FLASH_B      - 0x1108_0000 to 0x110F_FE00....in the 0x200 steps
*                                            Info Block A - 0x1180_0000
*                                            Info Block B - 0x1188_0000
*                   Write_buff              | pointer to buffer to be written
*   Return value:    None.
============================================================================================================================*/

__ramfunc  void CM41xFlash_WriteROW(unsigned char Flash_Controller, unsigned int ROW_ADDR, unsigned int* Write_buff)
{
    unsigned int loop_count;
  
    CM41xFlash_EnterCommandMode(Flash_Controller);

    if(Flash_Controller == FLASH_CONTROLLER_A)
    {
      //update Row page
      *pREG_FLC0_ADDR = ROW_ADDR ; 
       CM41xFlash_IssueCommand(FLASH_CONTROLLER_A, START_PROGRAM_CMD, POLL_COMMAND_STATUS); 

      for(loop_count=0; loop_count < FLASH_ROW_SIZE; loop_count+=8, ROW_ADDR+=8)
      {
        *pREG_FLC0_DATA0 = *Write_buff++; 
        *pREG_FLC0_DATA1 = *Write_buff++; 
        *pREG_FLC0_ADDR = ROW_ADDR ; 
         CM41xFlash_IssueCommand(FLASH_CONTROLLER_A, PROGRAM_CMD, POLL_COMMAND_STATUS); 
      }      
    }
    else if(Flash_Controller == FLASH_CONTROLLER_B)
    {
      //update Row page
      *pREG_FLC1_ADDR = ROW_ADDR ; 
       CM41xFlash_IssueCommand(FLASH_CONTROLLER_B, START_PROGRAM_CMD, POLL_COMMAND_STATUS); 

      for(loop_count=0; loop_count < FLASH_ROW_SIZE; loop_count+=8, ROW_ADDR+=8)
      {
        *pREG_FLC1_DATA0 = *Write_buff++; 
        *pREG_FLC1_DATA1 = *Write_buff++; 
        *pREG_FLC1_ADDR = ROW_ADDR ; 
         CM41xFlash_IssueCommand(FLASH_CONTROLLER_B, PROGRAM_CMD, POLL_COMMAND_STATUS); 
      }   
    }
    else
        Error_function();
          
    CM41xFlash_IssueCommand(Flash_Controller, END_PROGRAM_CMD, POLL_COMMAND_STATUS); 

    // exit command mode
    CM41xFlash_ExitCommandMode(Flash_Controller);
}


/*===========================================================================================================================
*	Function name:  CM41xFlash_WriteWord
*	Description:	Writes 4-byte word to flash
*	Parameter                               | Description                                       | Valid values                              |
*                   Flash_Controller        | Flash controller to be entered into command mode  | FLASH_CONTROLLER_A or FLASH_CONTROLLER_A
*                   WORD_ADDR: Address of ROW to be written.       
*                              Valid values: FLASH_A      - 0x1100_0000 to 0x1107_FFFC
*                                            FLASH_B      - 0x1108_0000 to 0x110F_FFFC
*                                            Info Block A - 0x1180_0000 to 0x1180_0FFC
*                                            Info Block B - 0x1188_0000 to 0x1188_0FFC
*                   Write_buff              | data to be written
*   Return value:    None.
============================================================================================================================*/

__ramfunc  void CM41xFlash_WriteWord(unsigned char Flash_Controller, unsigned int WORD_ADDR, unsigned int Write_Data)
{
    unsigned int DATA0, DATA1;
    
//    CM41xFlash_ResetController(Flash_Controller);
    CM41xFlash_EnterCommandMode(Flash_Controller);
    
    switch(Flash_Controller)
    {
        case FLASH_CONTROLLER_A:

          *pREG_FLC0_ADDR = WORD_ADDR ; 
          CM41xFlash_IssueCommand(FLASH_CONTROLLER_A, READ_CMD, POLL_COMMAND_STATUS); 
          DATA0 = *pREG_FLC0_DATA0 ;
          DATA1 = *pREG_FLC0_DATA1 ;

          if(WORD_ADDR & 0x4)
          {
            *pREG_FLC0_DATA0 = DATA0;
            *pREG_FLC0_DATA1 = Write_Data;
          }
          else
          {
            *pREG_FLC0_DATA0 = Write_Data;
            *pREG_FLC0_DATA1 = DATA1;
          }
            
          //update Row page
          *pREG_FLC0_ADDR = WORD_ADDR ; 
           CM41xFlash_IssueCommand(FLASH_CONTROLLER_A, START_PROGRAM_CMD, POLL_COMMAND_STATUS); 

          *pREG_FLC0_ADDR = WORD_ADDR ; 
           CM41xFlash_IssueCommand(FLASH_CONTROLLER_A, PROGRAM_CMD, POLL_COMMAND_STATUS); 
          
        break;  

        case FLASH_CONTROLLER_B:
 
          *pREG_FLC1_ADDR = WORD_ADDR ; 
          CM41xFlash_IssueCommand(FLASH_CONTROLLER_B, READ_CMD, POLL_COMMAND_STATUS); 
          DATA0 = *pREG_FLC1_DATA0 ;
          DATA1 = *pREG_FLC1_DATA1 ;

          if(WORD_ADDR & 0x4)
          {
            *pREG_FLC1_DATA0 = DATA0;
            *pREG_FLC1_DATA1 = Write_Data;
          }
          else
          {
            *pREG_FLC1_DATA0 = Write_Data;
            *pREG_FLC1_DATA1 = DATA1;
          }
            
          //update Row page
          *pREG_FLC1_ADDR = WORD_ADDR ; 
           CM41xFlash_IssueCommand(FLASH_CONTROLLER_B, START_PROGRAM_CMD, POLL_COMMAND_STATUS); 

          *pREG_FLC1_ADDR = WORD_ADDR ; 
           CM41xFlash_IssueCommand(FLASH_CONTROLLER_B, PROGRAM_CMD, POLL_COMMAND_STATUS); 
        break;  

   		default:
			printf("\nInvalid Flash Controller ID\n");
			Error_function();
		break;
    }
    
    CM41xFlash_IssueCommand(Flash_Controller, END_PROGRAM_CMD, POLL_COMMAND_STATUS); 
            
    //  exit command mode
    CM41xFlash_ExitCommandMode(Flash_Controller);
}


__ramfunc  void CM41xFlash_Write64bitWord_withECC(unsigned char Flash_Controller, unsigned int WORD_ADDR, unsigned int Write_Data0, unsigned int Write_Data1, unsigned char ECC_Byte)
{
    unsigned int DATA0, DATA1;
   unsigned char IRFWEN, IRFREN;
    CM41xFlash_SetInfoBlockProp(Flash_Controller, WORD_ADDR, &IRFWEN, &IRFREN);
    
//    CM41xFlash_ResetController(Flash_Controller);
    CM41xFlash_EnterCommandMode(Flash_Controller);
    
    switch(Flash_Controller)
    {
        case FLASH_CONTROLLER_A:
/*
          *pREG_FLC0_ADDR = WORD_ADDR ; 
          CM41xFlash_IssueCommand(FLASH_CONTROLLER_A, READ_CMD, POLL_COMMAND_STATUS); 
          DATA0 = *pREG_FLC0_DATA0 ;
          DATA1 = *pREG_FLC0_DATA1 ;

          if(WORD_ADDR & 0x4)
          {
            *pREG_FLC0_DATA0 = DATA0;
            *pREG_FLC0_DATA1 = Write_Data;
          }
          else
          {
            *pREG_FLC0_DATA0 = Write_Data;
            *pREG_FLC0_DATA1 = DATA1;
          }
*/
          *pREG_SPU1_EMODE3 = 0x79D1EA18;
          *pREG_SPU1_EMODE3 = 0xB22B248E;
          *pREG_TESYS1_ENG_MODE_CFG0= 0x4000; // FIXME Replace with bitmask
          
          *pREG_FLC0_CTL |= 0x10;
          *pREG_FLC0_ADDR  = WORD_ADDR ; 
          *pREG_FLC0_DATA0 = Write_Data0;
          *pREG_FLC0_DATA1 = Write_Data1;
          if(ECC_Byte == 0)
            *pREG_FLC0_ECC   = CM41xFlash_CalcECC(Write_Data0, Write_Data1);            
          else    
            *pREG_FLC0_ECC   = ECC_Byte;
           CM41xFlash_IssueCommand(FLASH_CONTROLLER_A, START_PROGRAM_CMD, POLL_COMMAND_STATUS); 

          *pREG_FLC0_ADDR = WORD_ADDR ; 
           CM41xFlash_IssueCommand(FLASH_CONTROLLER_A, PROGRAM_CMD, POLL_COMMAND_STATUS); 
          
          *pREG_FLC0_CTL &= ~0x10;
          *pREG_TESYS1_ENG_MODE_CFG0= 0x0000;
          *pREG_SPU1_EMODE3 = 0x001;
          *pREG_TESYS1_ENG_MODE_CFG0= 0x0000; // FIXME Replace with bitmask
            
        break;  

        case FLASH_CONTROLLER_B:
/* 
          *pREG_FLC1_ADDR = WORD_ADDR ; 
          CM41xFlash_IssueCommand(FLASH_CONTROLLER_B, READ_CMD, POLL_COMMAND_STATUS); 
          DATA0 = *pREG_FLC1_DATA0 ;
          DATA1 = *pREG_FLC1_DATA1 ;

          if(WORD_ADDR & 0x4)
          {
            *pREG_FLC1_DATA0 = DATA0;
            *pREG_FLC1_DATA1 = Write_Data;
          }
          else
          {
            *pREG_FLC1_DATA0 = Write_Data;
            *pREG_FLC1_DATA1 = DATA1;
          }
*/
          *pREG_SPU1_EMODE3 = 0x79D1EA18;
          *pREG_SPU1_EMODE3 = 0xB22B248E;
          *pREG_TESYS1_ENG_MODE_CFG0= 0x4000; // FIXME Replace with bitmask
          *pREG_FLC1_CTL |= 0x10;

          *pREG_FLC1_ADDR = WORD_ADDR ; 
           CM41xFlash_IssueCommand(FLASH_CONTROLLER_B, START_PROGRAM_CMD, POLL_COMMAND_STATUS); 

          *pREG_FLC1_ADDR = WORD_ADDR ; 
          *pREG_FLC1_DATA0 = Write_Data0;
          *pREG_FLC1_DATA1 = Write_Data1;
          if(ECC_Byte == 0)
            *pREG_FLC1_ECC   = CM41xFlash_CalcECC(Write_Data0, Write_Data1);            
          else    
            *pREG_FLC1_ECC   = ECC_Byte;           
          CM41xFlash_IssueCommand(FLASH_CONTROLLER_B, PROGRAM_CMD, POLL_COMMAND_STATUS); 

          *pREG_FLC1_CTL &= ~0x10;
          *pREG_TESYS1_ENG_MODE_CFG0= 0x0000;
          *pREG_SPU1_EMODE3 = 0x001;
          *pREG_TESYS1_ENG_MODE_CFG0= 0x0000; // FIXME Replace with bitmask

        break;  

   		default:
			printf("\nInvalid Flash Controller ID\n");
			Error_function();
		break;
    }
    
    CM41xFlash_IssueCommand(Flash_Controller, END_PROGRAM_CMD, POLL_COMMAND_STATUS); 
            
    //  exit command mode
    CM41xFlash_ExitCommandMode(Flash_Controller);
     
    CM41xFlash_RestoreInfoBlockProp(Flash_Controller, IRFWEN, IRFREN);

}
/*===========================================================================================================================
*	Function name:  CM41xFlash_SetInfoBlockProp
*	Description:	This function sets or clears IRFWEN or IRFREN bit of Flash controller, depending on FLASH_ADDR parameter passed.
*                   If FLASH_ADDR is within Info_block address, IRFWEN or IRFREN bit will be set depending on RD/WR operation defined in 'Access type'. 
*                       else corresponding bit will be cleared. 
*                   The original value of this bit is returned, so that CM41xFlash_RestoreInfoBlockProp() can restore the original value at the end of operation.
*	Parameter                               | Description                                       | Valid values                              |
*                   Flash_Controller        | Flash controller to be entered into command mode  | FLASH_CONTROLLER_A or FLASH_CONTROLLER_A
*                   FLASH_ADDR: 
*                              Valid values: FLASH_A      - 0x1100_0000 to 0x1107_FFFC
*                                            FLASH_B      - 0x1108_0000 to 0x110F_FFFC
*                                            Info Block A - 0x1180_0000 to 0x1180_0FFC
*                                            Info Block B - 0x1188_0000 to 0x1188_0FFC
*   Return value:   Original value of IRFWEN and IRFREN bit of Flash controller
============================================================================================================================*/

__ramfunc  void CM41xFlash_SetInfoBlockProp(unsigned char Flash_Controller, unsigned int FLASH_ADDR, unsigned char* IRFWEN, unsigned char* IRFREN)
{
    switch(Flash_Controller)
    {
        case FLASH_CONTROLLER_A:

            // If the address is within info block
            if(FLASH_ADDR & 0x00800000)
            {
              if(*pREG_FLC0_CTL & BITM_FLC_CTL_IRFWEN)
                *IRFWEN = 1;
              else
                *IRFWEN = 0;

              if(*pREG_FLC0_CTL & BITM_FLC_CTL_IRFREN)
                *IRFREN = 1;
              else
                *IRFREN = 0;
              
              *pREG_FLC0_CTL |= BITM_FLC_CTL_IRFWEN;  
              *pREG_FLC0_CTL |= BITM_FLC_CTL_IRFREN;  
            }
            else // If address is not within info block
            {
              if(*pREG_FLC0_CTL & BITM_FLC_CTL_IRFWEN)
                *IRFWEN = 1;
              else
                *IRFWEN = 0;

              if(*pREG_FLC0_CTL & BITM_FLC_CTL_IRFREN)
                *IRFREN = 1;
              else
                *IRFREN = 0;
              
              *pREG_FLC0_CTL &= ~BITM_FLC_CTL_IRFWEN;  
              *pREG_FLC0_CTL &= ~BITM_FLC_CTL_IRFREN;  
            }
          
        break; 

        case FLASH_CONTROLLER_B:

            // If the address is within info block
            if(FLASH_ADDR & 0x00800000)
            {
              if(*pREG_FLC1_CTL & BITM_FLC_CTL_IRFWEN)
                *IRFWEN = 1;
              else
                *IRFWEN = 0;

              if(*pREG_FLC1_CTL & BITM_FLC_CTL_IRFREN)
                *IRFREN = 1;
              else
                *IRFREN = 0;
              
              *pREG_FLC1_CTL |= BITM_FLC_CTL_IRFWEN;  
              *pREG_FLC1_CTL |= BITM_FLC_CTL_IRFREN;  
            }
            else // If address is not within info block
            {
              if(*pREG_FLC1_CTL & BITM_FLC_CTL_IRFWEN)
                *IRFWEN = 1;
              else
                *IRFWEN = 0;

              if(*pREG_FLC1_CTL & BITM_FLC_CTL_IRFREN)
                *IRFREN = 1;
              else
                *IRFREN = 0;
              
              *pREG_FLC1_CTL &= ~BITM_FLC_CTL_IRFWEN;  
              *pREG_FLC1_CTL &= ~BITM_FLC_CTL_IRFREN;  
            }

          break;          
          
   		default:
			printf("\nInvalid Flash Controller ID\n");
			Error_function();
		break;
    }   
}


/*===========================================================================================================================
*	Function name:  CM41xFlash_RestoreInfoBlockProp
*	Description:	This function restores the IRFWEN or IRFREN bit  which was modified in the CM41xFlash_SetInfoBlockProp fucntion.
*                   Ideally the returned value from CM41xFlash_SetInfoBlockProp function should be passed as Info_Block_RW_EN parameter to this function
*	Parameter                               | Description                                       | Valid values                              |
*                   Flash_Controller        | Flash controller to be entered into command mode  | FLASH_CONTROLLER_A or FLASH_CONTROLLER_A
*                   IRFWEN                  | 0 or 1...should be same as value returned from CM41xFlash_SetInfoBlockProp
*                   IRFREN                  | 0 or 1...should be same as value returned from CM41xFlash_SetInfoBlockProp
*   Return value:   None
============================================================================================================================*/

__ramfunc  void CM41xFlash_RestoreInfoBlockProp(unsigned char Flash_Controller, unsigned char IRFWEN, unsigned char IRFREN)
{
    switch(Flash_Controller)
    {
        case FLASH_CONTROLLER_A:

           if(IRFWEN == 1)
             *pREG_FLC0_CTL |= BITM_FLC_CTL_IRFWEN;  
           else
             *pREG_FLC0_CTL &= ~BITM_FLC_CTL_IRFWEN;

           if(IRFREN == 1)
             *pREG_FLC0_CTL |= BITM_FLC_CTL_IRFREN;  
           else
             *pREG_FLC0_CTL &= ~BITM_FLC_CTL_IRFREN;               
        
        break;

        case FLASH_CONTROLLER_B:

           if(IRFWEN == 1)
             *pREG_FLC1_CTL |= BITM_FLC_CTL_IRFWEN;  
           else
             *pREG_FLC1_CTL &= ~BITM_FLC_CTL_IRFWEN;

           if(IRFREN == 1)
             *pREG_FLC1_CTL |= BITM_FLC_CTL_IRFREN;  
           else
             *pREG_FLC1_CTL &= ~BITM_FLC_CTL_IRFREN;               
 
        break;
          
   		default:
			printf("\nInvalid Flash Controller ID\n");
			Error_function();
		break;
    }           
}


/*===========================================================================================================================
*	Function name:  CM41xFlash_SetPerformanceMode
*	Description:	This function sets the performance mode (A/B/C/E) of flash depending on core clock frequencies.
*	Parameter                               | Description                    
*                   core_clock              | Core clock of processor in MHz 
*   Return value:   None
============================================================================================================================*/
#if 0
__ramfunc void CM41xFlash_SetPerformanceMode(unsigned int core_clock)
{
  if(core_clock <= 75000000)
  {
    *pREG_FLC0_PRFCTL = (*pREG_FLC0_PRFCTL & (~BITM_FLC_PRFCTL_PMODE)) | ENUM_FLC_PRFCTL_A ; 
    *pREG_FLC1_PRFCTL = (*pREG_FLC1_PRFCTL & (~BITM_FLC_PRFCTL_PMODE)) | ENUM_FLC_PRFCTL_A ; 
  }
  else if(core_clock <= 150000000)
  {
    *pREG_FLC0_PRFCTL = (*pREG_FLC0_PRFCTL & (~BITM_FLC_PRFCTL_PMODE)) | ENUM_FLC_PRFCTL_B ; 
    *pREG_FLC1_PRFCTL = (*pREG_FLC1_PRFCTL & (~BITM_FLC_PRFCTL_PMODE)) | ENUM_FLC_PRFCTL_B ; 
  }
  else if(core_clock <= 225000000)
  {
    *pREG_FLC0_PRFCTL = (*pREG_FLC0_PRFCTL & (~BITM_FLC_PRFCTL_PMODE)) | ENUM_FLC_PRFCTL_C ; 
    *pREG_FLC1_PRFCTL = (*pREG_FLC1_PRFCTL & (~BITM_FLC_PRFCTL_PMODE)) | ENUM_FLC_PRFCTL_C ; 
  }
  else if(core_clock <= 240000000)
  {
    *pREG_FLC0_PRFCTL = (*pREG_FLC0_PRFCTL & (~BITM_FLC_PRFCTL_PMODE)) | ENUM_FLC_PRFCTL_E ; 
    *pREG_FLC1_PRFCTL = (*pREG_FLC1_PRFCTL & (~BITM_FLC_PRFCTL_PMODE)) | ENUM_FLC_PRFCTL_E ; 
  }
}
#endif
/*===========================================================================================================================
*	Function name:  Error function
*	Description:	This is error function, which will be called when  there is error in the flash status bits or if incorrect
*                       parameter is passed to the function.
*	Parameter       None
*   Return value:   None
============================================================================================================================*/

__ramfunc  void Error_function(void)
{
  printf("\nError\n");
  
  while(1)
    asm("nop;");
}
