/*************************************************************************************************************************
									File Details
**************************************************************************************************************************
	File name 							: CGUInit.c
	Purpose								: Has functions to initialize the CGU and CDU
	Author								: Nabeel
	VDSP version used for testing		: ..
										  Version:
                                          Build id:
	Hardware used						: N/A
	Connection details 					: N/A
	Guidelines for change				: N/A
**************************************************************************************************************************/
#include "CGUInit.h"


__ramfunc void CGU_Init(int iMultiplier, \
				  int bDivideFrequency, \
				  int iCCLKSel, \
				  int iDDCLKSel, \
				  int iSCLK0Sel, \
				  int iSCLK1Sel, \
				  int iSysCLKSel, \
				  int iOutClock, \
				  int iClkOut)
{
	CGU_Settings clk_setting;
        
	clk_setting.ctl_MSEL    = iMultiplier;
	clk_setting.ctl_DF      = bDivideFrequency;
	clk_setting.div_CSEL    = iCCLKSel;
	clk_setting.div_SYSSEL  = iSysCLKSel;
	clk_setting.div_S0SEL   = iSCLK0Sel;
	clk_setting.div_S1SEL   = iSCLK1Sel;
	clk_setting.div_DSEL    = iDDCLKSel;
	clk_setting.div_OSEL    = iOutClock;

        
        //*pREG_CGU0_PLLCTL |= BITM_CGU_PLLCTL_PLLBPCL;
        //__DSB();
	//cgu_mode();
	CGU_Init_All(0, 0, &clk_setting, iClkOut);
        
#ifdef ADI_ADSP_CM41X_M4 	
        unsigned int core_clock = (((CLKIN_CM40x)*iMultiplier)/(bDivideFrequency + 1))/iCCLKSel;
	CM41xFlash_SetPerformanceMode(core_clock);
#endif
        
}


/**************************************************************************************************************************
	Function name	:   CGU_Init
    Description		:   This function initializes the CGU to configure different clocks in system.

	Arguments		:	Parameter			 |	Description		 						| Valid values
						uiCguNo		 	     |  Instance of CGU         				| 0, 1
						uiClkinsel		     |  CLKIN source for CGU    				| 0, 1
						*pClocks		     |  Pointer to Structure which holds        | MSEL, DF, CSEL, SYSSEL, S0SEL,
                                                different Clock multipliers and           S1SEL, DSEL, OSEL values
                                                dividers
                        uiClkoutsel          |  Selects one of the clock to CLKOUT      | Processor dependent
	Return value	:   Success/Error code.
**************************************************************************************************************************/
__ramfunc uint32_t CGU_Init_All(uint32_t uiCguNo, uint32_t uiClkinsel, CGU_Settings *pClocks,  uint32_t uiClkoutsel)
{
    uint32_t result = 0;
#ifdef SLATER
    CGU_Freq_Check_Round(pClocks, 30);  // Second paramenter is CLKIN value. It is ignored inside this function and is taken from CGUInit.h file
    Active_To_Fullon(0);
    uint32_t dActionFlags = BITM_ROM_SYSCTRL_CGU_CTL | BITM_ROM_SYSCTRL_CGU_DIV ;
	if(*pREG_CGU0_STAT & BITM_CGU_STAT_PLLBP)   // Check if processor is in Active mode
	{
		if(!(*pREG_CGU0_STAT & BITM_CGU_STAT_PLLEN) )  // Check if PLL is disabled
		{
			ActivePLLDis_To_Active(0);
		}
		Active_To_Fullon(0);
	}  
    CGU0_write(dActionFlags, pClocks, uiClkoutsel);
    return result;
#endif

#if defined(TESLA_M0) || defined(TESLA_M4)
    CGU_Freq_Check_Round(pClocks, 25);  // Second paramenter is CLKIN value. It is ignored inside this function and is taken from CGUInit.h file
    uint32_t dActionFlags = BITM_ROM_SYSCTRL_CGU_CTL | BITM_ROM_SYSCTRL_CGU_DIV ;
	if(*pREG_CGU0_STAT & BITM_CGU_STAT_PLLBP)   // Check if processor is in Active mode
	{
		if(!(*pREG_CGU0_STAT & BITM_CGU_STAT_PLLEN) )  // Check if PLL is disabled
		{
			ActivePLLDis_To_Active(0);
		}
		Active_To_Fullon(0);
	}  
    CGU0_write(dActionFlags, pClocks, uiClkoutsel);
    return result;
#endif

}


/* @brief       Performs a CGU write operation.
* @details     This function performs the write operation to configure new frequencies as requested
*  			by the user
* @param [in]  dActionFlags     Specifies the action requested by the user
* @param [in]  *pSyscontrol     Pointer to the ROM structure
* @param [in]  *reserved        Reserved for future use
* @return       Status
*				- #0			 													Success
* 				- #BITM_ROM_SYSCTRL_READ_ERR | BITM_ROM_SYSCTRL_CGU_STAT_ERR		Error due to PLL disabled
*				- #BITM_CGU_STAT_WDIVERR | BITM_CGU_STAT_WDFMSERR                   CGU status error occured
*/
__ramfunc uint32_t CGU0_write(uint32_t dActionFlags, CGU_Settings *pCGU_Settings, uint32_t uiClkOutSel)
{
	uint32_t dNewCguDiv;																							/* variable to hold the new CGU-DIV value */
	uint8_t dPllBypassed  = (*pREG_CGU0_STAT & BITM_CGU_STAT_PLLBP);												/* Variable to hold the status of PLL Bypass */
	uint8_t dPllEnabled   = (*pREG_CGU0_STAT & BITM_CGU_STAT_PLLEN);												/* Variable to hold the status of PLL Enable */

	/* Check if write to CGU_DIV register is requested */
	if( dActionFlags & BITM_ROM_SYSCTRL_CGU_DIV)
	{
		dNewCguDiv = 	((OSEL(pCGU_Settings->div_OSEL)) 		|
						(SYSSEL(pCGU_Settings->div_SYSSEL )) 	|
						(CSEL(pCGU_Settings->div_CSEL )) 		|
						(S0SEL(pCGU_Settings->div_S0SEL)) 		|
						(S1SEL(pCGU_Settings->div_S1SEL)) 		|
						(DSEL(pCGU_Settings->div_DSEL )))		&
						(~BITM_CGU_DIV_LOCK);

		if(!dPllBypassed)																							/* Check if PLL is not bypassed */
		{
			dNewCguDiv 	|= (BITM_CGU_DIV_UPDT)  ;																	/* Set the Align and Update bits */
		}

		if( !(dActionFlags & BITM_ROM_SYSCTRL_CGU_AUTO_DIS))														/* Check if disabling the auto handling of ALGN and UPDT fields is requested */
		{
			if(!dPllBypassed)
			{
				dNewCguDiv 	|= (BITM_CGU_DIV_ALGN | BITM_CGU_DIV_UPDT)  ;											/* Set the Align and Update bits */
			}

			if( ((dNewCguDiv) & (DIV_MASK)) == ((*pREG_CGU0_DIV) & (DIV_MASK)) )									/* Check if new values involve a frequecy change */
			{
				dActionFlags 	&=  (~BITM_ROM_SYSCTRL_CGU_DIV);													/* Clear the Flag for write to DIV register */
			}
		}
	}

	if( dActionFlags & BITM_ROM_SYSCTRL_CGU_CTL)																	/* Check if write to CGU_CTL register is requested */
	{
		uint32_t dNewCguCtl  = (MSEL( pCGU_Settings->ctl_MSEL )| DF( pCGU_Settings->ctl_DF ) );

		if( ((dNewCguCtl) & (CTL_MASK)) == ((*pREG_CGU0_CTL) & (CTL_MASK)) )										/* Check if new values involve a frequecy change */
		{
			dActionFlags 	&=  (~BITM_ROM_SYSCTRL_CGU_CTL);														/* Clear the Flag for write to CTL register */
		}
	}

	Program_Cgu0(dActionFlags, dPllBypassed, dPllEnabled, dNewCguDiv, pCGU_Settings);						/* Program the CTL and DIV registers */

	*pREG_CGU0_CLKOUTSEL  = uiClkOutSel;

	return 0;
}

/**
* @brief       Program the CGU
* @details     This function the CGU_CTL and CGU_CTL registers
* @param [in]  dActionFlags     Specifies the action requested by the user
* @param [in]  *pSyscontrol     Pointer to the ROM structure
* @param [in]  *reserved        Reserved for future use
* @return       None
*/
__ramfunc void Program_Cgu0(uint32_t dActionFlags, uint32_t dPllBypassed, uint32_t dPllEnabled, uint32_t dNewCguDiv, CGU_Settings *pCGU_Settings)
{
	if( !(dActionFlags & BITM_ROM_SYSCTRL_CGU_CTL) )																/* Check if CGU_CTL write is not requested */
	{
		if( dActionFlags & BITM_ROM_SYSCTRL_CGU_DIV)																/* Check if CGU_DIV needs to be written */
		{
			if( (!dPllBypassed) && (dPllEnabled) )																	/* Check if PLL is not bypassed or disabled */
			{
				while(*pREG_CGU0_STAT & BITM_CGU_STAT_CLKSALGN);      												/* Check if clocks are aligned before modyfying the DIV register */
				/* !!!!!! No Status bit to know error condition  !!!!!  */
				/*  !!!!!! Decide if this loop needs to be broken !!!!!  */
			}
                                        
			*pREG_CGU0_DIV = dNewCguDiv;	/* Update the new Divider values */
                        *pREG_CGU0_CTL =  (MSEL( pCGU_Settings->ctl_MSEL ) | DF( pCGU_Settings->ctl_DF ) ) & (~BITM_CGU_CTL_LOCK);
                        
			if(  (!dPllBypassed) && (dPllEnabled) )																	/* Check if PLL is not bypassed or disabled */
			{
				while(*pREG_CGU0_STAT & BITM_CGU_STAT_CLKSALGN);													/* Wait till clocks are aligned */
			}
		}
	}/* end of CGU_DIV Write */

	else																											/* Update both CGU_CTL and CGU_DIV registers */
	{
		if(  (!dPllBypassed) && (dPllEnabled) )																		/* Check if PLL is not bypassed or disabled */
		{
			while( !((*pREG_CGU0_STAT & (BITM_CGU_STAT_PLLEN | BITM_CGU_STAT_PLOCK | BITM_CGU_STAT_CLKSALGN) ) ==((BITM_CGU_STAT_PLLEN | BITM_CGU_STAT_PLOCK) & (~BITM_CGU_STAT_CLKSALGN))));
		} 																											/* Check if PLL is enabled , PLL is not locking and Clocks are aligned */

		if( dActionFlags & BITM_ROM_SYSCTRL_CGU_DIV)
		{
                  *pREG_CGU0_DIV = dNewCguDiv & (~BITM_CGU_DIV_ALGN) & (~BITM_CGU_DIV_UPDT); //dNewCguDiv & (~BITM_CGU_DIV_ALGN) & (~BITM_CGU_DIV_UPDT);  /* Clear the Align and Update bit, since CTL is also written */
		}
    
		*pREG_CGU0_CTL =  (MSEL( pCGU_Settings->ctl_MSEL ) | DF( pCGU_Settings->ctl_DF ) ) & (~BITM_CGU_CTL_LOCK);

		if(  (!dPllBypassed) && (dPllEnabled) )																		/* Check if PLL is not bypassed or disabled */
		{
			while( !((*pREG_CGU0_STAT & (BITM_CGU_STAT_PLOCK | BITM_CGU_STAT_CLKSALGN) ) == (BITM_CGU_STAT_PLOCK & (~BITM_CGU_STAT_CLKSALGN))));
		} 																											/* Wait till PLL is locked and clocks are aligned */
	}/* end of CGU_DIV and CGU_CTL Write */
}

/**************************************************************************************************************************
	Function name	:   CGU_Freq_Check_Round
    Description		:   This function check round the user frequencies if they are out of spec

	Arguments		:	Parameter			 |	Description		 						| Valid values
                        *pClocks		     |  Pointer to Structure which holds        | MSEL, DF, CSEL, SYSSEL, S0SEL,
                                                different Clock multipliers and           S1SEL, DSEL, OSEL values
                                                dividers

	Return value	:   None.
**************************************************************************************************************************/
__ramfunc void CGU_Freq_Check_Round(CGU_Settings *pClocks, float uiClkinValue)
{
#ifdef SLATER
    float uiClkinVal   = (float)CLKIN_CM40x/1000000;

    uint32_t uiPllclk_Max   = PLLCLK_MAX_CM40x;
    uint32_t uiCclk_Max     = CCLK_MAX_CM40x;
    uint32_t uiSysclk_Max   = SYSCLK_MAX_CM40x;
    //uint32_t uiS0clk_Max    = 0;
    //uint32_t uiS1clk_Max    = 0;
    uint32_t uiDclk_Max     = DCLK_MAX_CM40x;
#endif

#if defined(TESLA_M0) || defined(TESLA_M4)
    float uiClkinVal   = (float)CLKIN_CM41x/1000000;

    uint32_t uiPllclk_Max   = PLLCLK_MAX_CM41x;
    uint32_t uiCclk_Max     = CCLK_MAX_CM41x;
    uint32_t uiSysclk_Max   = SYSCLK_MAX_CM41x;
    uint32_t uiS0clk_Max    = S0CLK_MAX_CM41x;
    //uint32_t uiS1clk_Max    = 0;
    uint32_t uiDclk_Max     = DCLK_MAX_CM41x;
#endif


    float Pll_Clock    = uiClkinVal*(pClocks->ctl_MSEL)/((pClocks->ctl_DF) + 1);
    float Core_Clock   = Pll_Clock/pClocks->div_CSEL;
    float Sys_Clock    = Pll_Clock/pClocks->div_SYSSEL;
    float S0_Clock     = Sys_Clock/pClocks->div_S0SEL;
    float S1_Clock     = Sys_Clock/pClocks->div_S1SEL;
    float Ddr_Clock    = Pll_Clock/pClocks->div_DSEL;
    float Out_Clock    = Pll_Clock/pClocks->div_OSEL;

    if( Pll_Clock > uiPllclk_Max)                        /* If PLL clock is out of spec reduce MSEL to make PLL clcok within the spec */
    {
        while(Pll_Clock > uiPllclk_Max)
        {
            pClocks->ctl_MSEL--;
            Pll_Clock    = uiClkinVal*(pClocks->ctl_MSEL)/((pClocks->ctl_DF) + 1);
        }

        // Recalculate all the clocks
        Pll_Clock    = uiClkinVal*(pClocks->ctl_MSEL)/((pClocks->ctl_DF) + 1);
        Core_Clock   = Pll_Clock/pClocks->div_CSEL;
        Sys_Clock    = Pll_Clock/pClocks->div_SYSSEL;
        S0_Clock     = Sys_Clock/pClocks->div_S0SEL;
        S1_Clock     = Sys_Clock/pClocks->div_S1SEL;
        Ddr_Clock    = Pll_Clock/pClocks->div_DSEL;
        Out_Clock    = Pll_Clock/pClocks->div_OSEL;

    }

    if( Core_Clock > uiCclk_Max)                         /* If Core clock is out of spec reduce MSEL to make Core clock within the spec */
    {
        while(Core_Clock > uiCclk_Max)
        {
            pClocks->ctl_MSEL--;
            Pll_Clock    = uiClkinVal*(pClocks->ctl_MSEL)/((pClocks->ctl_DF) + 1);
            Core_Clock   = Pll_Clock/pClocks->div_CSEL;
        }

        // Recalculate all the clocks
		 Pll_Clock    = uiClkinVal*(pClocks->ctl_MSEL)/((pClocks->ctl_DF) + 1);
		 Core_Clock   = Pll_Clock/pClocks->div_CSEL;
		 Sys_Clock    = Pll_Clock/pClocks->div_SYSSEL;
		 S0_Clock     = Sys_Clock/pClocks->div_S0SEL;
		 S1_Clock     = Sys_Clock/pClocks->div_S1SEL;
		 Ddr_Clock    = Pll_Clock/pClocks->div_DSEL;
		 Out_Clock    = Pll_Clock/pClocks->div_OSEL;
    }

    if( Sys_Clock > uiSysclk_Max)                         /* If Sys clock is out of spec reduce MSEL to make Sys clock within the spec */
    {
        while(Sys_Clock > uiSysclk_Max)
        {
            pClocks->ctl_MSEL--;
            Pll_Clock    = uiClkinVal*(pClocks->ctl_MSEL)/((pClocks->ctl_DF) + 1);
            Sys_Clock    = Pll_Clock/pClocks->div_SYSSEL;
        }

        // Recalculate all the clocks
		 Pll_Clock    = uiClkinVal*(pClocks->ctl_MSEL)/((pClocks->ctl_DF) + 1);
		 Core_Clock   = Pll_Clock/pClocks->div_CSEL;
		 Sys_Clock    = Pll_Clock/pClocks->div_SYSSEL;
		 S0_Clock     = Sys_Clock/pClocks->div_S0SEL;
		 S1_Clock     = Sys_Clock/pClocks->div_S1SEL;
		 Ddr_Clock    = Pll_Clock/pClocks->div_DSEL;
		 Out_Clock    = Pll_Clock/pClocks->div_OSEL;
    }

#if defined (TESLA_M0) || defined(TESLA_M4)

    if( S0_Clock > uiS0clk_Max)                         /* If S0 clock is out of spec reduce MSEL to make S0 clock within the spec */
    {
        while(S0_Clock > uiS0clk_Max)
        {
            pClocks->ctl_MSEL--;
            Pll_Clock    = uiClkinVal*(pClocks->ctl_MSEL)/((pClocks->ctl_DF) + 1);
            S0_Clock     = Sys_Clock/pClocks->div_S0SEL;
        }

        // Recalculate all the clocks
		 Pll_Clock    = uiClkinVal*(pClocks->ctl_MSEL)/((pClocks->ctl_DF) + 1);
		 Core_Clock   = Pll_Clock/pClocks->div_CSEL;
		 Sys_Clock    = Pll_Clock/pClocks->div_SYSSEL;
		 S0_Clock     = Sys_Clock/pClocks->div_S0SEL;
		 S1_Clock     = Sys_Clock/pClocks->div_S1SEL;
		 Ddr_Clock    = Pll_Clock/pClocks->div_DSEL;
		 Out_Clock    = Pll_Clock/pClocks->div_OSEL;
    }
#endif

// Note: Slater and Tesla does not have S1 Clock
/*
    if( S1_Clock > uiS1clk_Max)                         // If S1 clock is out of spec reduce MSEL to make S0 clock within the spec
    {
    	while(S1_Clock > uiS1clk_Max)
        {
            pClocks->ctl_MSEL--;
            Pll_Clock    = uiClkinVal*(pClocks->ctl_MSEL)/((pClocks->ctl_DF) + 1);
            S1_Clock     = Sys_Clock/pClocks->div_S1SEL;
        }

        // Recalculate all the clocks
		 Pll_Clock    = uiClkinVal*(pClocks->ctl_MSEL)/((pClocks->ctl_DF) + 1);
		 Core_Clock   = Pll_Clock/pClocks->div_CSEL;
		 Sys_Clock    = Pll_Clock/pClocks->div_SYSSEL;
		 S0_Clock     = Sys_Clock/pClocks->div_S0SEL;
		 S1_Clock     = Sys_Clock/pClocks->div_S1SEL;
		 Ddr_Clock    = Pll_Clock/pClocks->div_DSEL;
		 Out_Clock    = Pll_Clock/pClocks->div_OSEL;
    }
*/

    if( Ddr_Clock > uiDclk_Max)                         // If Ddr_Clock is out of spec reduce MSEL to make DDR clock within the spec
    {
        while(Ddr_Clock > uiDclk_Max)
        {
            pClocks->ctl_MSEL--;
            Pll_Clock    = uiClkinVal*(pClocks->ctl_MSEL)/((pClocks->ctl_DF) + 1);
            Ddr_Clock    = Pll_Clock/pClocks->div_DSEL;
        }

        // Recalculate all the clocks
		 Pll_Clock    = uiClkinVal*(pClocks->ctl_MSEL)/((pClocks->ctl_DF) + 1);
		 Core_Clock   = Pll_Clock/pClocks->div_CSEL;
		 Sys_Clock    = Pll_Clock/pClocks->div_SYSSEL;
		 S0_Clock     = Sys_Clock/pClocks->div_S0SEL;
		 S1_Clock     = Sys_Clock/pClocks->div_S1SEL;
		 Ddr_Clock    = Pll_Clock/pClocks->div_DSEL;
		 Out_Clock    = Pll_Clock/pClocks->div_OSEL;
    }

}


/**************************************************************************************************************************
	Function name	:   Fullon_To_Active
    Description		:   This function changes the Operating mode from FullOn to Active
	Arguments		:	Parameter			 |	Description		 						| Valid values
						uiCGU_ID           	 |  CGU number                      	    | 0,1

	Return value	:   None
**************************************************************************************************************************/
__ramfunc void Fullon_To_Active(unsigned int uiCGU_ID)
{
    if(uiCGU_ID == 0)
    {
    	uint32_t oldCGU0_STAT ;

        while(1)                                    // Wait Loop to ensure pre-switch conditions
        {
            oldCGU0_STAT = *pREG_CGU0_STAT;
            if((oldCGU0_STAT & BITM_CGU_STAT_PLLEN) && ~(oldCGU0_STAT & BITM_CGU_STAT_PLLBP) && ~(oldCGU0_STAT & BITM_CGU_STAT_CLKSALGN))
            {   break; }
        }
        
#ifdef SLATER
        *pREG_DPM0_CTL = BITM_DPM_CTL_PLLBPST;      // Enable PLL ByPass
#endif
        
#if defined(TESLA_M0) || defined(TESLA_M4)
        *pREG_CGU0_PLLCTL = BITM_CGU_PLLCTL_PLLBPST;    // Enable PLL ByPass
#endif
        

	    while(1)                                        // Wait loop to ensure post-switch exit conditions
        {
            oldCGU0_STAT = *pREG_CGU0_STAT;
            if (oldCGU0_STAT & BITM_CGU_STAT_PLLBP)
            {   break; }
        }
    }
}

/**************************************************************************************************************************
	Function name	:   Active_To_Fullon
    Description		:   This function changes the Operating mode from Active to FullOn
	Arguments		:	Parameter			 |	Description		 						| Valid values
						uiCGU_ID           	 |  CGU number                      	    | 0,1

	Return value	:   None
**************************************************************************************************************************/
__ramfunc void Active_To_Fullon(unsigned int uiCGU_ID)
{
    if(uiCGU_ID == 0)
    {
        uint32_t oldCGU0_STAT;

        while(1)                                        // Wait Loop to ensure pre-switch conditions
        {
            oldCGU0_STAT = *pREG_CGU0_STAT;
            if((oldCGU0_STAT & BITM_CGU_STAT_PLLEN) && (oldCGU0_STAT & BITM_CGU_STAT_PLLBP))
            {   break; }
        }

#ifdef SLATER
        *pREG_DPM0_CTL = BITM_DPM_CTL_PLLBPCL;    // PLL Disable
#endif
       
#if defined(TESLA_M0) || defined(TESLA_M4)
        *pREG_CGU0_PLLCTL = BITM_CGU_PLLCTL_PLLBPCL;   // PLL Disable
#endif
        
        while(1)   // Wait Loop to ensure post-switch conditions
        {
            oldCGU0_STAT = *pREG_CGU0_STAT;
            if ((oldCGU0_STAT & BITM_CGU_STAT_PLLEN) && ~(oldCGU0_STAT & BITM_CGU_STAT_PLLBP) && ~(oldCGU0_STAT & BITM_CGU_STAT_CLKSALGN))
            {   break; }
        }
    }
}

/**************************************************************************************************************************
	Function name	:   Active_To_ActivePLLDis
    Description		:   This function changes the Operating mode from Active to Active with PLL Disabled
	Arguments		:	Parameter			 |	Description		 						| Valid values
						uiCGU_ID           	 |  CGU number                      	    | 0,1

	Return value	:   None
**************************************************************************************************************************/
__ramfunc void Active_To_ActivePLLDis(unsigned int uiCGU_ID)
{
    if(uiCGU_ID == 0)
    {
        uint32_t  oldCGU0_STAT;

        while(1)                                    // Wait Loop to ensure pre-switch conditions
        {
            oldCGU0_STAT = *pREG_CGU0_STAT;
            if((oldCGU0_STAT & BITM_CGU_STAT_PLLEN) && (oldCGU0_STAT & BITM_CGU_STAT_PLLBP))
            {  break;  }
        }

#ifdef SLATER
        *pREG_DPM0_CTL = BITM_DPM_CTL_PLLDIS;
#endif
        
#if defined(TESLA_M0) || defined(TESLA_M4)
        *pREG_CGU0_PLLCTL = BITM_CGU_PLLCTL_PLLDIS;
#endif
          
        while(1)                                    // Wait Loop to ensure post-switch conditions
        {
            oldCGU0_STAT = *pREG_CGU0_STAT;
            if(~(oldCGU0_STAT & BITM_CGU_STAT_PLLEN) && ~(oldCGU0_STAT & BITM_CGU_STAT_CLKSALGN) && (oldCGU0_STAT & BITM_CGU_STAT_PLLBP))
            {  break;  }
        }
    }
}

/**************************************************************************************************************************
	Function name	:   ActivePLLDis_To_Active
    Description		:   This function changes the Operating mode from Active with PLL Disabled to Active
	Arguments		:	Parameter			 |	Description		 						| Valid values
						uiCGU_ID           	 |  CGU number                      	    | 0,1

	Return value	:   None
**************************************************************************************************************************/
__ramfunc void ActivePLLDis_To_Active(unsigned int uiCGU_ID)
{
    if(uiCGU_ID == 0)
    {
    	uint32_t  oldCGU0_STAT;

        while(1)                                    // Wait Loop to ensure pre-switch conditions
        {
            oldCGU0_STAT = *pREG_CGU0_STAT;
            if((oldCGU0_STAT & BITM_CGU_STAT_PLLBP) && ~(oldCGU0_STAT & BITM_CGU_STAT_PLLEN))
            {  break;   }
        }

#ifdef SLATER
        *pREG_DPM0_CTL |= ((0 << BITP_DPM_CTL_PLLDIS) & BITM_DPM_CTL_PLLDIS);  // PLL Enable
#endif
        
#if defined(TESLA_M0) || defined(TESLA_M4)
        *pREG_CGU0_PLLCTL = BITM_CGU_PLLCTL_PLLEN;  // PLL Enable
#endif
        
        while (1)
        {
            oldCGU0_STAT = *pREG_CGU0_STAT;
            if((oldCGU0_STAT & BITM_CGU_STAT_PLLBP) && (oldCGU0_STAT & BITM_CGU_STAT_PLLEN))
            {  break;   }
        }
    }
}


__ramfunc void cgu_mode()
{
  *pREG_CGU0_PLLCTL |= BITM_CGU_PLLCTL_PLLBPST;
  __DSB();
  *pREG_CGU0_PLLCTL |= BITM_CGU_PLLCTL_PLLDIS;
  __DSB();
  
  while((*pREG_CGU0_STAT & BITM_CGU_STAT_CLKSALGN));
  
  *pREG_CGU0_PLLCTL |= BITM_CGU_PLLCTL_PLLEN;
  __DSB();
  
  while((*pREG_CGU0_STAT & BITM_CGU_STAT_CLKSALGN));

  *pREG_CGU0_PLLCTL |= BITM_CGU_PLLCTL_PLLBPCL;
  __DSB();
   while((*pREG_CGU0_STAT & BITM_CGU_STAT_CLKSALGN));
}

#ifdef ADI_ADSP_CM41X_M4 
__ramfunc void CM41xFlash_SetPerformanceMode(unsigned int core_clock)
{
  if(core_clock <= 75000000)
  {
    *pREG_FLC0_PRFCTL = (*pREG_FLC0_PRFCTL & (~BITM_FLC_PRFCTL_PMODE)) | ENUM_FLC_PRFCTL_A ; 
    *pREG_FLC1_PRFCTL = (*pREG_FLC1_PRFCTL & (~BITM_FLC_PRFCTL_PMODE)) | ENUM_FLC_PRFCTL_A ; 
  }
  else if(core_clock <= 150000000)
  {
    *pREG_FLC0_PRFCTL = (*pREG_FLC0_PRFCTL & (~BITM_FLC_PRFCTL_PMODE)) | ENUM_FLC_PRFCTL_B ; 
    *pREG_FLC1_PRFCTL = (*pREG_FLC1_PRFCTL & (~BITM_FLC_PRFCTL_PMODE)) | ENUM_FLC_PRFCTL_B ; 
  }
  else if(core_clock <= 225000000)
  {
    *pREG_FLC0_PRFCTL = (*pREG_FLC0_PRFCTL & (~BITM_FLC_PRFCTL_PMODE)) | ENUM_FLC_PRFCTL_C ; 
    *pREG_FLC1_PRFCTL = (*pREG_FLC1_PRFCTL & (~BITM_FLC_PRFCTL_PMODE)) | ENUM_FLC_PRFCTL_C ; 
  }
  else if(core_clock <= 240000000)
  {
    *pREG_FLC0_PRFCTL = (*pREG_FLC0_PRFCTL & (~BITM_FLC_PRFCTL_PMODE)) | ENUM_FLC_PRFCTL_E ; 
    *pREG_FLC1_PRFCTL = (*pREG_FLC1_PRFCTL & (~BITM_FLC_PRFCTL_PMODE)) | ENUM_FLC_PRFCTL_E ; 
  }
}
#endif