/*********************************************************************************

Copyright(c) 2011 Analog Devices, Inc. All Rights Reserved.

This software is proprietary and confidential.  By using this software you agree
to the terms of the associated Analog Devices License Agreement.

*********************************************************************************/
/*!
* @file      adi_tmr_data_bf6xx.c
*
* @brief     Timer service global API implementation.
*
* @details
*            This is a source file which contains data related to timer service
*            for BF6XX processor family.
*/

#ifndef __ADI_TMR_DATA_BF6XX__
#define __ADI_TMR_DATA_BF6XX__

#include "adi_tmr_def_bf6xx.h"

#ifdef _MISRA_RULES
#pragma diag(push)
#pragma diag(suppress:misra_rule_8_5:"Allow variables to be defined in the source file.  \
                                      Misra treats this as header file as it is included \
                                      in other source file")
#pragma diag(suppress:misra_rule_11_4:"Allow tapecasting between different pointer types")
#pragma diag(suppress:misra_rule_2_4:"Allows doxygen addtogroup block")
#endif

/** @addtogroup Timer_Service GP Timer Service
*  @{
*/

/* On dual-core ADSP-CM41x, we have two timer blocks, for a total of 16 timers:
	TIMER0 block (8 timers) primairly intended for the M0 core, and
	TIMER1 block (8 timers) primairly intended for the M4 code.
	There is overhead/delay when one core tries to use the other's timer block.
	(Note, that the TRU supports smooth routing of timer events across cores.)
	For the M0 core, we map the the timer driver to TIMER0, whereas
	for the M4 core, we map the the timer driver to TIMER1,
	thereby using the same timer IDs across both cores (seperate executables).
	For future referance, we may need to support all 16 timers on both cores
	which would require revising the adi_tmr_Init() API to take a timer block ID,
	but for now we treat the two timer blocks as mutually exclusive.
*/
#if defined (ADI_ADSP_CM41X) && defined (ADI_ADSP_CM41X_M4)

/* TIMER1 Mappings for Tesla M4 core */
#define INTR_TIMERn_STAT          INTR_TIMER1_STAT
#define pREG_TIMERn_BCAST_DLY     pREG_TIMER1_BCAST_DLY
#define pREG_TIMERn_BCAST_PER     pREG_TIMER1_BCAST_PER
#define pREG_TIMERn_BCAST_WID     pREG_TIMER1_BCAST_WID
#define pREG_TIMERn_DATA_ILAT     pREG_TIMER1_DATA_ILAT
#define pREG_TIMERn_DATA_IMSK     pREG_TIMER1_DATA_IMSK
#define pREG_TIMERn_ERR_TYPE      pREG_TIMER1_ERR_TYPE
#define pREG_TIMERn_RUN           pREG_TIMER1_RUN
#define pREG_TIMERn_RUN_CLR       pREG_TIMER1_RUN_CLR
#define pREG_TIMERn_RUN_SET       pREG_TIMER1_RUN_SET
#define pREG_TIMERn_STAT_ILAT     pREG_TIMER1_STAT_ILAT
#define pREG_TIMERn_STAT_IMSK     pREG_TIMER1_STAT_IMSK
#define pREG_TIMERn_STOP_CFG_CLR  pREG_TIMER1_STOP_CFG_CLR
#define pREG_TIMERn_STOP_CFG_SET  pREG_TIMER1_STOP_CFG_SET
#define pREG_TIMERn_TRG_IE        pREG_TIMER1_TRG_IE
#define pREG_TIMERn_TRG_MSK       pREG_TIMER1_TRG_MSK

/* Table of available timers in the ADI_ADSP_CM41X M4 processor */
static ADI_TMR_DEV  gaTimers[] =
{
    /* Timer 0 */
    {
        (ADI_TMR_REGS_PTR) pREG_TIMER1_TMR0_CFG,    /* Timer register address base */
        (uint32_t)INTR_TIMER1_TMR0,                 /* Timer data interrupt SID */
        0u,                                         /* Timer ID */
        NULL                                        /* Pointer to timer instance Data */
    },
    /* Timer 1 */
    {
        (ADI_TMR_REGS_PTR) pREG_TIMER1_TMR1_CFG,    /* Timer register address base */
        (uint32_t)INTR_TIMER1_TMR1,                 /* Timer data interrupt SID */
        1u,                                         /* Timer ID */
        NULL                                        /* Pointer to timer instance Data */
    },
    /* Timer 2 */
    {
        (ADI_TMR_REGS_PTR) pREG_TIMER1_TMR2_CFG,    /* Timer register address base */
        (uint32_t)INTR_TIMER1_TMR2,                 /* Timer data interrupt SID */
        2u,                                         /* Timer ID */
        NULL                                        /* Pointer to timer instance Data */
    },
    /* Timer 3 */
    {
        (ADI_TMR_REGS_PTR) pREG_TIMER1_TMR3_CFG,    /* Timer register address base */
        (uint32_t)INTR_TIMER1_TMR3,                 /* Timer data interrupt SID */
        3u,                                         /* Timer ID */
        NULL                                        /* Pointer to timer instance Data */
    },
    /* Timer 4 */
    {
        (ADI_TMR_REGS_PTR) pREG_TIMER1_TMR4_CFG,    /* Timer register address base */
        (uint32_t)INTR_TIMER1_TMR4,                 /* Timer data interrupt SID */
        4u,                                         /* Timer ID */
        NULL                                        /* Pointer to timer instance Data */
    },
    /* Timer 5 */
    {
        (ADI_TMR_REGS_PTR) pREG_TIMER1_TMR5_CFG,    /* Timer register address base */
        (uint32_t)INTR_TIMER1_TMR5,                 /* Timer data interrupt SID */
        5u,                                         /* Timer ID */
        NULL                                        /* Pointer to timer instance Data */
    },
    /* Timer 6 */
    {
        (ADI_TMR_REGS_PTR) pREG_TIMER1_TMR6_CFG,    /* Timer register address base */
        (uint32_t)INTR_TIMER1_TMR6,                 /* Timer data interrupt SID */
        6u,                                         /* Timer ID */
        NULL                                        /* Pointer to timer instance Data */
    },
    /* Timer 7 */
    {
       (ADI_TMR_REGS_PTR) pREG_TIMER1_TMR7_CFG,     /* Timer register address base */
       (uint32_t)INTR_TIMER1_TMR7,                  /* Timer data interrupt SID */
        7u,                                         /* Timer ID */
        NULL                                        /* Pointer to timer instance Data */
    }
};

#else

/* Default TIMER0 Mappings for all other cores (including Tesla M0) */
#define INTR_TIMERn_STAT          INTR_TIMER0_STAT
#define pREG_TIMERn_BCAST_DLY     pREG_TIMER0_BCAST_DLY
#define pREG_TIMERn_BCAST_PER     pREG_TIMER0_BCAST_PER
#define pREG_TIMERn_BCAST_WID     pREG_TIMER0_BCAST_WID
#define pREG_TIMERn_DATA_ILAT     pREG_TIMER0_DATA_ILAT
#define pREG_TIMERn_DATA_IMSK     pREG_TIMER0_DATA_IMSK
#define pREG_TIMERn_ERR_TYPE      pREG_TIMER0_ERR_TYPE
#define pREG_TIMERn_RUN           pREG_TIMER0_RUN
#define pREG_TIMERn_RUN_CLR       pREG_TIMER0_RUN_CLR
#define pREG_TIMERn_RUN_SET       pREG_TIMER0_RUN_SET
#define pREG_TIMERn_STAT_ILAT     pREG_TIMER0_STAT_ILAT
#define pREG_TIMERn_STAT_IMSK     pREG_TIMER0_STAT_IMSK
#define pREG_TIMERn_STOP_CFG_CLR  pREG_TIMER0_STOP_CFG_CLR
#define pREG_TIMERn_STOP_CFG_SET  pREG_TIMER0_STOP_CFG_SET
#define pREG_TIMERn_TRG_IE        pREG_TIMER0_TRG_IE
#define pREG_TIMERn_TRG_MSK       pREG_TIMER0_TRG_MSK

/* Table of available timers in the processor */
static ADI_TMR_DEV  gaTimers[] =
{
    /* Timer 0 */
    {
        (ADI_TMR_REGS_PTR) pREG_TIMER0_TMR0_CFG,    /* Timer register address base */
        (uint32_t)INTR_TIMER0_TMR0,                 /* Timer data interrupt SID */
        0u,                                         /* Timer ID */
        NULL                                        /* Pointer to timer instance Data */
    },
    /* Timer 1 */
    {
        (ADI_TMR_REGS_PTR) pREG_TIMER0_TMR1_CFG,    /* Timer register address base */
        (uint32_t)INTR_TIMER0_TMR1,                 /* Timer data interrupt SID */
        1u,                                         /* Timer ID */
        NULL                                        /* Pointer to timer instance Data */
    },
    /* Timer 2 */
    {
        (ADI_TMR_REGS_PTR) pREG_TIMER0_TMR2_CFG,    /* Timer register address base */
        (uint32_t)INTR_TIMER0_TMR2,                 /* Timer data interrupt SID */
        2u,                                         /* Timer ID */
        NULL                                        /* Pointer to timer instance Data */
    },
    /* Timer 3 */
    {
        (ADI_TMR_REGS_PTR) pREG_TIMER0_TMR3_CFG,    /* Timer register address base */
        (uint32_t)INTR_TIMER0_TMR3,                 /* Timer data interrupt SID */
        3u,                                         /* Timer ID */
        NULL                                        /* Pointer to timer instance Data */
    },
    /* Timer 4 */
    {
        (ADI_TMR_REGS_PTR) pREG_TIMER0_TMR4_CFG,    /* Timer register address base */
        (uint32_t)INTR_TIMER0_TMR4,                 /* Timer data interrupt SID */
        4u,                                         /* Timer ID */
        NULL                                        /* Pointer to timer instance Data */
    },
    /* Timer 5 */
    {
        (ADI_TMR_REGS_PTR) pREG_TIMER0_TMR5_CFG,    /* Timer register address base */
        (uint32_t)INTR_TIMER0_TMR5,                 /* Timer data interrupt SID */
        5u,                                         /* Timer ID */
        NULL                                        /* Pointer to timer instance Data */
    },
    /* Timer 6 */
    {
        (ADI_TMR_REGS_PTR) pREG_TIMER0_TMR6_CFG,    /* Timer register address base */
        (uint32_t)INTR_TIMER0_TMR6,                 /* Timer data interrupt SID */
        6u,                                         /* Timer ID */
        NULL                                        /* Pointer to timer instance Data */
    },
    /* Timer 7 */
    {
       (ADI_TMR_REGS_PTR) pREG_TIMER0_TMR7_CFG,     /* Timer register address base */
       (uint32_t)INTR_TIMER0_TMR7,                  /* Timer data interrupt SID */
        7u,                                         /* Timer ID */
        NULL                                        /* Pointer to timer instance Data */
    }
};
#endif

/*@}*/

#ifdef _MISRA_RULES
#pragma diag(pop)
#endif

#endif /* __ADI_TMR_DATA_BF6XX__ */
