/*!
 *****************************************************************************
 * @file:    startup.c
 * @brief:   Interrupt table and default handlers for ADSP-CM41x for M0 Core
 * @version: $Revision: 30937 $
 * @date:    $Date: 2015-05-06 15:19:05 -0400 (Wed, 06 May 2015) $
 *-----------------------------------------------------------------------------
 *
Copyright (c) 2010 Analog Devices, Inc.

All rights reserved.

Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:
  - Redistributions of source code must retain the above copyright notice,
    this list of conditions and the following disclaimer.
  - Redistributions in binary form must reproduce the above copyright notice,
    this list of conditions and the following disclaimer in the documentation
    and/or other materials provided with the distribution.
  - Modified versions of the software must be conspicuously marked as such.
  - This software is licensed solely and exclusively for use with processors
    manufactured by or for Analog Devices, Inc.
  - This software may not be combined or merged with other code in any manner
    that would cause the software to become subject to terms and conditions
    which differ from those listed here.
  - Neither the name of Analog Devices, Inc. nor the names of its
    contributors may be used to endorse or promote products derived
    from this software without specific prior written permission.
  - The use of this software may or may not infringe the patent rights of one
    or more patent holders.  This license does not release you from the
    requirement that you obtain separate licenses from these patent holders
    to use this software.

THIS SOFTWARE IS PROVIDED BY ANALOG DEVICES, INC. AND CONTRIBUTORS "AS IS" AND ANY
EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, NON-INFRINGEMENT,
TITLE, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN
NO EVENT SHALL ANALOG DEVICES, INC. OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, PUNITIVE OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, DAMAGES ARISING OUT OF CLAIMS OF INTELLECTUAL
PROPERTY RIGHTS INFRINGEMENT; PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *****************************************************************************/

#include <stdio.h> // for NULL
#include "startup.h"

#ifdef ADI_MT_SUPPORT
/*
 * Micrium uCOS-III redefines these handler names.  If the application
 * uses uCOS-III the macro ADI_MT_SUPPORT must be added to the project
 * preprocessor options.
 */
#define PendSV_Handler  OS_CPU_PendSVHandler
#define SysTick_Handler OS_CPU_SysTickHandler
#endif

//*****************************************************************************
//
// Local helpers for consistent ISR prototypes and implementations.
//
//*****************************************************************************
#define WEAK_PROTOTYPE(x) WEAK_PROTO ( void x (void)) ATTRIBUTE_INTERRUPT ;

#ifdef HANDLERS_IN_FLASH
// Locate the handlers in flash. The default linker .icf file copies "ro" code
// to RAM. To place the handlers in flash the .icf file must be modified to
// not copy "ro" code:
//   initialize by copy { rw };
#define WEAK_FUNCTION(x)  WEAK_FUNC  ( void x (void)) { while(1); }
#else
// Note: because the reset handler (ResetISR) is where the startup lives,
// the linker can't to initialize it because its needed for startup.  That
// causes the linker to mark the entire module as un-initializable and would
// result in all handlers being located in flash (along with the reset vector).
// Marking the bulk of the interrupt handlers as "__ramfunc" tells the linker
// its OK to locate them in RAM as part of the initialization process.  It
// puts them into a special ".textrw" section which is placed explicitly
// into the RAM_CODE region by the icf file (in addition to ".text").
#define WEAK_FUNCTION(x)  WEAK_FUNC  ( __ramfunc void x (void)) { while(1); }
#endif

//*****************************************************************************
//
// Forward declaration of implimented exception handlers.
//
//*****************************************************************************
WEAK_PROTOTYPE( ResetISR                           )
WEAK_PROTOTYPE( NmiSR                              )
WEAK_PROTOTYPE( HardFault_Handler                  )
WEAK_PROTOTYPE( SVC_Handler                        )
WEAK_PROTOTYPE( PendSV_Handler                     )
WEAK_PROTOTYPE( SysTick_Handler                    )

//*****************************************************************************
//
// Forward declaration of the all the programmable device interrupt handlers.
//
//*****************************************************************************
WEAK_PROTOTYPE( si0Trap                            )
WEAK_PROTOTYPE( CTI1_EVT0_Handler                  )
WEAK_PROTOTYPE( M0P_SRAM_EERR_Handler              )
WEAK_PROTOTYPE( si3Trap                            )
WEAK_PROTOTYPE( MBOX0_PORT0_EERR_Handler           )
WEAK_PROTOTYPE( si5Trap                            )
WEAK_PROTOTYPE( si6Trap                            )
WEAK_PROTOTYPE( si7Trap                            )
WEAK_PROTOTYPE( si8Trap                            )
WEAK_PROTOTYPE( WDOG0_EXP_Handler                  )
WEAK_PROTOTYPE( si10Trap                           )
WEAK_PROTOTYPE( si11Trap                           )
WEAK_PROTOTYPE( si12Trap                           )
WEAK_PROTOTYPE( ADCC0_ERR_Handler                  )
WEAK_PROTOTYPE( PINT0_BLOCK_Handler                )
WEAK_PROTOTYPE( si15Trap                           )
WEAK_PROTOTYPE( si16Trap                           )
WEAK_PROTOTYPE( si17Trap                           )
WEAK_PROTOTYPE( si18Trap                           )
WEAK_PROTOTYPE( si19Trap                           )
WEAK_PROTOTYPE( TRU0_INT1_Handler                  )
WEAK_PROTOTYPE( TRU1_INT4_Handler                  )
WEAK_PROTOTYPE( si22Trap                           )
WEAK_PROTOTYPE( ADCC0_TMR0_EVT_Handler             )
WEAK_PROTOTYPE( ADCC0_TMR1_EVT_Handler             )
WEAK_PROTOTYPE( si25Trap                           )
WEAK_PROTOTYPE( UART0_STAT_Handler                 )
WEAK_PROTOTYPE( HAE0_STAT_Handler                  )
WEAK_PROTOTYPE( si28Trap                           )
WEAK_PROTOTYPE( TWI0_DATA_Handler                  )
WEAK_PROTOTYPE( MBOX0_PORT0_MSG_Handler            )
WEAK_PROTOTYPE( M0P_SOFT_INT0_Handler              )

WEAK_PROTOTYPE( RESERVED_VECTOR                    )

/*
    Stack size considerations for RTOS environments:

    In the Cortex-M class processors, the hardware has two stacks: the main stack
    and the process stack.  The main stack is used at reset and by system exception
    handlers.  In simple applications (non-RTOS) you may only use this stack.  In
    more complex systems that use an RTOS you have a split between the main stack
    which is used by the kernel and exception handlers and the process stack which
    is used by the tasks and programmable interrupts (interrupts stack on top of
    the currently active task, so tasks stacks should be allocated for worst-case
    interrupt scenarios).  In the uCOS case, the process stack is the one that gets
    context-switched and used by the tasks, where we store the registers, etc.

    As RTOS tasks specify their own stack area, a minimal size of 4 for the process
    stack in order to retain symbols, etc., seems OK and we would not need any more.
    The main stack needs to be big enough to initialize the RTOS, run interrupts etc.

    As always, stack and heap size optimizations is a balancing act between usage
    and available memory.  It is entirely application-dependant.

*/


// IVT typedefs
typedef void  ( *IntFunc )( void );
typedef union { IntFunc __fun; void * __ptr; } IntVector;

// import linker symbol
#pragma segment="CSTACK"

// Interrupt Vector Table
#pragma location = ".intvec"
__root const IntVector __vector_table[] =
{
    // grab stack pointer "end" address from ICF file
    { .__ptr = __sfe( "CSTACK" ) },

    // exception mappings...
    {ResetISR                          },  // -15
    {NmiSR                             },  // -14
    {HardFault_Handler                 },  // -13
    {RESERVED_VECTOR                   },  // -12
    {RESERVED_VECTOR                   },  // -11
    {RESERVED_VECTOR                   },  // -10
    {RESERVED_VECTOR                   },  // -09
    {RESERVED_VECTOR                   },  // -08
    {RESERVED_VECTOR                   },  // -07
    {RESERVED_VECTOR                   },  // -06
    {SVC_Handler                       },  // -05
    {RESERVED_VECTOR                   },  // -04
    {RESERVED_VECTOR                   },  // -03
    {PendSV_Handler                    },  // -02  (Micrium overrides weak binding if using uCOS)
    {SysTick_Handler                   },  // -01  (Micrium overrides weak binding if using uCOS)

    // 32 programmable interrupt mappings...
    // Shared interrupts are initially assigned to a common trap.
    // Dedicated interrupts are initially assigned a unique trap.
    // Use adi_int_InstallHandler() to register real interrupt handlers
    // for both shared and dedicated interrupts.
    {si0Trap                         },  //   0  shared interrupt trap
    {CTI1_EVT0_Handler              },  //   1  Event 0
    {M0P_SRAM_EERR_Handler          },  //   2  M0P supervisor sram ecc error
    {si3Trap                         },  //   3  shared interrupt trap
    {MBOX0_PORT0_EERR_Handler       },  //   4  MBOX0 Port 0 ECC error
    {si5Trap                         },  //   5  shared interrupt trap
    {si6Trap                         },  //   6  shared interrupt trap
    {si7Trap                         },  //   7  shared interrupt trap
    {si8Trap                         },  //   8  shared interrupt trap
    {WDOG0_EXP_Handler              },  //   9  WDOG0 Expiration
    {si10Trap                         },  //  10  shared interrupt trap
    {si11Trap                         },  //  11  shared interrupt trap
    {si12Trap                         },  //  12  shared interrupt trap
    {ADCC0_ERR_Handler              },  //  13  ADCC0 ADC Error
    {PINT0_BLOCK_Handler            },  //  14  PINT0 Pin Interrupt Block
    {si15Trap                         },  //  15  shared interrupt trap
    {si16Trap                         },  //  16  shared interrupt trap
    {si17Trap                         },  //  17  shared interrupt trap
    {si18Trap                         },  //  18  shared interrupt trap
    {si19Trap                         },  //  19  shared interrupt trap
    {TRU0_INT1_Handler              },  //  20  TRU0 Interrupt request 1
    {TRU1_INT4_Handler              },  //  21  TRU1 Interrupt request 4
    {si22Trap                         },  //  22  shared interrupt trap
    {ADCC0_TMR0_EVT_Handler         },  //  23  ADCC0 Timer 0 Event Complete
    {ADCC0_TMR1_EVT_Handler         },  //  24  ADCC0 Timer 1 Event Complete
    {si25Trap                         },  //  25  shared interrupt trap
    {UART0_STAT_Handler             },  //  26  UART0 Status
    {HAE0_STAT_Handler              },  //  27  HAE0 Status
    {si28Trap                         },  //  28  shared interrupt trap
    {TWI0_DATA_Handler              },  //  29  TWI0 Data Interrupt
    {MBOX0_PORT0_MSG_Handler        },  //  30  MBOX0 Port 0 software interrupt
    {M0P_SOFT_INT0_Handler          },  //  31  MBOX0 Port 0 software interrupt
};


#ifndef DO_NOT_RELOCATE_IVT
/*
    A relocated IVT is requested.  Provision for IVT relocation
    to RAM during startup.  This allows for dynamic interrupt
    vector patching required by RTOS.  Places the relocated IVT
    at the start of RAM.  Note: the IVT placement is required
    to be next power-of-two of the vector table size.  Since the
    IVT includes 163 programmable interrupts, 15 system exception
    vectors plus the initial stack pointer, we require
    (76 + 6 + 1)*4 = 332 bytes to move the IVT, which rounds up
    to a next-power-of-two size of 512 (0x200) address boundary.

    Also note use of the "__no_init" attibute to force the
    relocated IVT into the ".noinit" section.  This prevents
    the CRTL startup sequence from initializing the relocated
    IVT after we have activated it during the reset vector.
    so that the CRTL does not clear it.
*/

/*
    Choose the new IVT address within the Supervisor SRAM for ideal interrupt
    latency (the default CSTACK block is 0x800 bytes at address 0x200f77f8, so a
    0x200 byte relocated vector table would want to be placed at something like
    0x20077578, but this is totally configurable.)  But since the default IVT
    placement at 0x20070000 is ALREADY in SRAM (and therefore patachable), there
    is probably never a good reason to move it... so this can all just be
    ignored (by setting DO_NOT_RELOCATE_IVT).
*/
#define RELOCATION_ADDRESS   (0x20077000)  /* allocate 512 bytes towards end of supervisor SRAM */
#define RELOCATION_ALIGNMENT      (0x100)  /* 8-bit aligned */
#define SIZEOF_IVT sizeof(__vector_table)
#if defined ( __ICCARM__ )
    #pragma data_alignment=RELOCATION_ALIGNMENT  /* IAR */
#elif defined (__CC_ARM)
    __align(RELOCATION_ALIGNMENT)                /* Keil */
#else
    #pragma message("WARNING: NO ALIGHMENT DEFINED FOR IVT RELOCATION")
#endif
/* reserve no-init aligned IVT space at top of RAM */
SECTION_PLACE(KEEP_VAR(__no_init IntVector __relocated_vector_table[SIZEOF_IVT/4]), RELOCATION_ADDRESS);

#endif //DO_NOT_RELOCATE_IVT


#ifdef __ARMCC_VERSION
// We want a warning if semi-hosting libraries are used.
#pragma import(__use_no_semihosting_swi)
#endif

//*****************************************************************************
//
// Application Entry point
//
//*****************************************************************************
extern int main(void);


//*****************************************************************************
// Symbols defined by the GNU linker.
//*****************************************************************************
#ifdef __GNUC__
extern unsigned long _etext;
extern unsigned long _data;
extern unsigned long _edata;
extern unsigned long _bss ;
extern unsigned long _ebss;
#endif

//*****************************************************************************
// IAR system startup functions.
//*****************************************************************************
#ifdef __ICCARM__
WEAK_PROTO( void __iar_init_core (void));
WEAK_PROTO( void __iar_init_vfp  (void));
extern      void __cmain();
#endif

//*****************************************************************************
// Function    : ResetISR (-15)
// Description : Reset event handler
//*****************************************************************************
WEAK_FUNC( void ResetISR (void)) {

    /*
        Note: the BOOT-ROM IVT is still active at this time, which sets
        the fault pin on any non-maskable exceptions.  After 1) the default
        IVT is copied to SRAM (below) and, 2) the interrupt handlers are
        copied (by the CRTL), we change the Interrupt Vector Table Offset
        Register (VTOR) in main as part of SystemInit().  Until this sequence
        is complete, the BOOT-ROM IVT remains active.
    */


    /* reset the main SP to clean up any leftovers from the boot rom. */
    __set_MSP( (uint32_t) __vector_table[0].__ptr );

#ifndef DO_NOT_RELOCATE_IVT
    int i;
    uint8_t *pSrc, *pDst;

    /* copy the IVT (avoid use of memcpy here so it does not become locked into flash) */
    for (i=0, pSrc=(uint8_t*)__vector_table, pDst=(uint8_t*)__relocated_vector_table; i<SIZEOF_IVT; i++)
        *pDst++ = *pSrc++;

#endif //DO_NOT_RELOCATE_IVT


#ifdef __GNUC__

  unsigned long *pulSrc, *pulDest;

    // Copy initialised data from flash into RAM
    pulSrc = &_etext;
    for(pulDest = &_data; pulDest < &_edata; )
    {
        *pulDest++ = *pulSrc++;
    }

    // Clear the bss segment
    for(pulDest = &_bss; pulDest < &_ebss; )
    {
        *pulDest++ = 0;
    }
    // Call application main directly.
    main();

#elif __ICCARM__

    // Call IAR system startup
    __iar_init_core();
    __iar_init_vfp();
    __cmain();

#else

    // Call application main directly.
    main();

#endif

    // Stick here if main returns
    while(1);
}


//*****************************************************************************
// Provide default (weakly bound) trapping handlers to be overridden as needed.
//
// NOTE: the SystemServices/DeviceDrivers typically install their own handlers
// to override these defaults using the "adi_int_InstallHandler()" API.  The
// default "weak" bindings here are intended to capture any "unhandled"
// interrupts and to allow the user to override them with custom handlers
// (that do not use the SystemServices/DeviceDrivers), if desired.
// In the case of custome user overrides, the user needs to also take into
// account that many interrupts are grouped and that distinguishing between
// the mupliple interrupts within each shared group needs to be also managed.
//*****************************************************************************
WEAK_FUNCTION( NmiSR                              )
WEAK_FUNCTION( HardFault_Handler                  )
WEAK_FUNCTION( SVC_Handler                        )
WEAK_FUNCTION( PendSV_Handler                     )
WEAK_FUNCTION( SysTick_Handler                    )

WEAK_FUNCTION( si0Trap                            )
WEAK_FUNCTION( CTI1_EVT0_Handler                  )
WEAK_FUNCTION( M0P_SRAM_EERR_Handler              )
WEAK_FUNCTION( si3Trap                            )
WEAK_FUNCTION( MBOX0_PORT0_EERR_Handler           )
WEAK_FUNCTION( si5Trap                            )
WEAK_FUNCTION( si6Trap                            )
WEAK_FUNCTION( si7Trap                            )
WEAK_FUNCTION( si8Trap                            )
WEAK_FUNCTION( WDOG0_EXP_Handler                  )
WEAK_FUNCTION( si10Trap                           )
WEAK_FUNCTION( si11Trap                           )
WEAK_FUNCTION( si12Trap                           )
WEAK_FUNCTION( ADCC0_ERR_Handler                  )
WEAK_FUNCTION( PINT0_BLOCK_Handler                )
WEAK_FUNCTION( si15Trap                           )
WEAK_FUNCTION( si16Trap                           )
WEAK_FUNCTION( si17Trap                           )
WEAK_FUNCTION( si18Trap                           )
WEAK_FUNCTION( si19Trap                           )
WEAK_FUNCTION( TRU0_INT1_Handler                  )
WEAK_FUNCTION( TRU1_INT4_Handler                  )
WEAK_FUNCTION( si22Trap                           )
WEAK_FUNCTION( ADCC0_TMR0_EVT_Handler             )
WEAK_FUNCTION( ADCC0_TMR1_EVT_Handler             )
WEAK_FUNCTION( si25Trap                           )
WEAK_FUNCTION( UART0_STAT_Handler                 )
WEAK_FUNCTION( HAE0_STAT_Handler                  )
WEAK_FUNCTION( si28Trap                           )
WEAK_FUNCTION( TWI0_DATA_Handler                  )
WEAK_FUNCTION( MBOX0_PORT0_MSG_Handler            )
WEAK_FUNCTION( M0P_SOFT_INT0_Handler              )

WEAK_FUNCTION( RESERVED_VECTOR                    )
