/*
  Programming the Oscillator and Voltage Monitors for CM41x

  This example shows how to setup and use the Oscillator Watchdog Unit, the
  Oscillator Comparator Unit and the Voltage Monitoring Unit.

  Analog Devices Inc.
  
*/
#include "CGUInit.h"
#include "GPIO_Safe_State_Init.h"
#include "MiscFunctions.h"
#include "OCU_Init.h"
#include "OSCWD_Init.h"
#include "VMU_Init.h"


//Comment out the #define to disable the modules
#define OSCWD           //Oscillator Watchdog
#define OCU             //Oscillator Comparator Unit
#define VMU             //Voltage Monitoring Unit
#define Enable_Interrupts

//Defines for the Oscillator Watchdog Unit.
#define FAULTPINDIS 0      //Fault Pin Disable
#define LOCK 0             //Lock OSCWDCTL register
#define HODF 21            //Watchdog Harmonic Lower Frequency Limit
#define HODEN 1            //Harmonic Oscillation Detection Enable
#define CNGEN 1            //Clock Not Good Enable
#define BOUF 0             //Bad Oscillator Frequency Limit    
#define BOUEN 1            //Bad Oscillator Frequency Limit Detection Enable      
#define FAULTEN 1          //Fault Enable
#define MONDIS 0           //OCU Monitor Disable

//Main Program 
int main()
{
  //Initialize the system and configure the clocks
  SystemInit();
  struct CGU_Settings Clock_Dividers = {32,0,4,10,1,1,10,5};      /* Initialize the structure */
  CGU_Init_All(0, 0, &Clock_Dividers, 11); 
  
  
// Oscillator Watchdog Config
#ifdef OSCWD
  osc_wd_Enable(FAULTPINDIS,LOCK,HODF,HODEN,CNGEN,BOUF,BOUEN,FAULTEN,MONDIS);
#endif
  
  
// Oscillator Comparator Unit Config
#ifdef OCU
  OCU_Init(1000000,50,50);
  
  *pREG_OCU0_CMONCNT = 96000000;
  *pREG_OCU0_LMONCNT = 1000000;
#ifdef Enable_Interrupts
  adi_nvic_EnableInt(INTR_OCU0_ERR, 1);
#endif
  OCU_Enable_Fault(FREQ_FAULT);
  OCU_Start_Auto();
  
  //LED2 Will turn on when OCU interrupt is triggered
  *pREG_PORTA_FER |= 0x0000;
  *pREG_PORTA_MUX |= 0x0000;
  *pREG_PORTA_DIR_SET = 0x2000;
  *pREG_PORTA_DATA = 0x0000;
 
#endif
  
  
// Voltage Monitoring Unit Config
#ifdef VMU
#ifdef Enable_Interrupts
   adi_nvic_EnableInt(INTR_VMU0_VDDINT_EVT, 1);
   adi_nvic_EnableInt(INTR_VMU0_VDDEXT_EVT, 1);
#endif
   
  *pREG_PORTB_FER|=0x0000f;                     // When mux[2i+1:2i] is written then fer[i] must be 0
  *pREG_PORTB_MUX|=0x00003C00;                  // mux PB 5,6 to VMU_TOUT
  *pREG_PORTB_FER|=0x0060;                      // setup PB 6,5 to be in periphial mode 
  *pREG_TEPADS1_PORTB_TRIPST = 0x00C3C000;      // set portb 7,8 to drive high in trip state, 11 to high as well for tin toggle
  *pREG_TEPADS1_PORTB_TRIPSEL = 0xff7f;         // set 7 to force_early, 8  and rest to force_late
  *pREG_PORTB_DIR_SET = 0x0180;                 // set 7/8 to outputs
  *pREG_PORTB_DIR_SET = 0x0800;                 // set 11 to input
  *pREG_PORTB_DATA = 0x0000;                    // write 0 to 7/8 
  
  //Enable VDD_EXT Monitoring
  VMU_Init(1,1,0,0,0,0);
  *pREG_TEPADS1_VMU_TRIPEN = 0x8;
  GPIO_Safe_State_Init(PORTB, TRISTATE, EARLY);
  
  //LED3 will turn on when VMU interrupt is triggered
 *pREG_PORTF_FER |= 0x0000;
 *pREG_PORTF_MUX |= 0x0000;
 *pREG_PORTF_DIR_SET = 0x0004;
 *pREG_PORTF_DATA = 0x0000;
  
#endif
  
  
  while(1){
    
  }
  
}

//Interrupt for OCU
void OCU0_ERR_Handler(void)
{
  if((*pREG_OCU0_STAT & BITM_OCU_STAT_FREQ_FAULT) >> BITP_OCU_STAT_FREQ_FAULT )
  {
    *pREG_PORTA_DATA = 0x2000;
    *pREG_OCU0_STAT &= 0xFFFFFFFE;
  }
}

//Interrupts for VMU
void VMU0_VDDINT_EVT_Handler(void){
  *pREG_PORTF_DATA = 0x0004;
}
void VMU0_VDDEXT_EVT_Handler(void){
  *pREG_PORTF_DATA = 0x0004;
}
